@@ -5,6 +5,10 @@
 #include "util/lists.h"
 #include "util/time.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct listbox_item;
 struct input_history;
 
@@ -13,22 +17,26 @@ struct global_history_item {
 
 	struct listbox_item *box_item;
 
-	unsigned char *title;
-	unsigned char *url;
+	char *title;
+	char *url;
 
 	time_t last_visit;
 };
 
 extern struct input_history global_history;
 
-extern unsigned char *gh_last_searched_title;
-extern unsigned char *gh_last_searched_url;
+extern char *gh_last_searched_title;
+extern char *gh_last_searched_url;
 
 extern struct module global_history_module;
 
 void delete_global_history_item(struct global_history_item *);
-struct global_history_item *get_global_history_item(unsigned char *);
-void add_global_history_item(unsigned char *, unsigned char *, time_t);
-int globhist_simple_search(unsigned char *, unsigned char *);
+struct global_history_item *get_global_history_item(char *);
+void add_global_history_item(char *, char *, time_t);
+int globhist_simple_search(char *, char *);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
