@@ -8,6 +8,12 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
+
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
 #if HAVE_LANGINFO_CODESET
 #include <langinfo.h>
 #endif
@@ -23,16 +29,26 @@
 #include <iconv.h>
 #endif
 
+#include <sys/ioctl.h>
+
+#ifdef HAVE_SYS_KD_H
+#include <sys/kd.h>
+#endif
+
 #include "elinks.h"
 
 #include "document/options.h"
 #include "intl/charsets.h"
+#ifdef CONFIG_OS_DOS
+#include "osdep/dos/dos.h"
+#endif
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/fastfind.h"
 #include "util/hash.h"
 #include "util/memory.h"
 #include "util/string.h"
+#include "osdep/osdep.h"
 
 
 /* Fix namespace clash on MacOS. */
@@ -51,8 +67,8 @@ struct table_entry {
 };
 
 struct codepage_desc {
-	unsigned char *name;
-	unsigned char *const *aliases;
+	const char *name;
+	const char **aliases;
 
  	/* The Unicode mappings of codepage bytes 0x80...0xFF.
  	 * (0x00...0x7F are assumed to be ASCII in all codepages.)
@@ -86,11 +102,10 @@ struct codepage_desc {
  * The non-inline declarations in charsets.h also make sure that the
  * compiler emits global definitions for the symbols so that the
  * functions can be called from other translation units.  */
-NONSTATIC_INLINE unsigned char *encode_utf8(unicode_val_T u);
-NONSTATIC_INLINE int utf8charlen(const unsigned char *p);
-NONSTATIC_INLINE int unicode_to_cell(unicode_val_T c);
-NONSTATIC_INLINE unicode_val_T utf8_to_unicode(unsigned char **string,
-					       const unsigned char *end);
+NONSTATIC_INLINE char *encode_utf8(unicode_val_T u);
+NONSTATIC_INLINE int utf8charlen(const char *p);
+NONSTATIC_INLINE unicode_val_T utf8_to_unicode(char **string,
+					       const char *end);
 
 static const char strings[256][2] = {
 	"\000", "\001", "\002", "\003", "\004", "\005", "\006", "\007",
@@ -148,7 +163,7 @@ free_translation_table(struct conv_table *p)
  * named array rather than a pointer so that it won't share storage
  * with any other string literal that happens to have the same
  * characters.  */
-static const unsigned char no_str[] = "*";
+static const char no_str[] = "*";
 
 static void
 new_translation_table(struct conv_table *p)
@@ -185,6 +200,50 @@ new_translation_table(struct conv_table *p)
 	}										\
 }											\
 
+/* list of unicode codepoints supported by the current terminal, if this
+ * information is available, otherwise size = -1 */
+
+struct {
+	int size;
+	unicode_val_T *list;
+} codepoints;
+
+static int
+is_codepoint_supported(unicode_val_T u)
+{
+	int first, last, middle;
+
+	if (codepoints.size == -1)
+		return 1;
+
+	first = 0;
+	last = codepoints.size - 1;
+
+	while (first <= last) {
+		middle = (last + first) / 2;
+		if (codepoints.list[middle] == u)
+			return u;
+		else if (codepoints.list[middle] > u)
+			last = middle - 1;
+		else
+			first = middle + 1;
+	}
+
+	return 0;
+}
+
+int
+codepoint_replacement(unicode_val_T u)
+{
+	int s;
+
+	if (is_codepoint_supported(u))
+		return -1;
+
+	BIN_SEARCH(unicode_7b, x, N_UNICODE_7B, u, s);
+	return s;
+}
+
 static const unicode_val_T strange_chars[32] = {
 0x20ac, 0x0000, 0x002a, 0x0000, 0x201e, 0x2026, 0x2020, 0x2021,
 0x005e, 0x2030, 0x0160, 0x003c, 0x0152, 0x0000, 0x0000, 0x0000,
@@ -195,7 +254,7 @@ static const unicode_val_T strange_chars[32] = {
 #define SYSTEM_CHARSET_FLAG 128
 #define is_cp_ptr_utf8(cp_ptr) ((cp_ptr)->aliases == aliases_utf8)
 
-const unsigned char *
+const char *
 u2cp_(unicode_val_T u, int to, enum nbsp_mode nbsp_mode)
 {
 	int j;
@@ -235,13 +294,21 @@ u2cp_(unicode_val_T u, int to, enum nbsp_mode nbsp_mode)
 	return no_str;
 }
 
-static unsigned char utf_buffer[7];
+static char utf_buffer[7];
 
-NONSTATIC_INLINE unsigned char *
+NONSTATIC_INLINE char *
 encode_utf8(unicode_val_T u)
 {
+	int s;
+
 	memset(utf_buffer, 0, 7);
 
+#ifdef CONFIG_CODEPOINT
+	if (!is_codepoint_supported(u)) {
+		BIN_SEARCH(unicode_7b, x, N_UNICODE_7B, u, s);
+		if (s != -1) return (char *)unicode_7b[s].s;
+	}
+#endif
 	if (u < 0x80)
 		utf_buffer[0] = u;
 	else if (u < 0x800)
@@ -287,16 +354,16 @@ static const char utf8char_len_tab[256] = {
 
 #ifdef CONFIG_UTF8
 NONSTATIC_INLINE int
-utf8charlen(const unsigned char *p)
+utf8charlen(const char *p)
 {
-	return p ? utf8char_len_tab[*p] : 0;
+	return p ? utf8char_len_tab[(unsigned char)*p] : 0;
 }
 
 int
-strlen_utf8(unsigned char **str)
+strlen_utf8(char **str)
 {
-	unsigned char *s = *str;
-	unsigned char *end = strchr((const char *)s, '\0');
+	char *s = *str;
+	char *end = strchr(s, '\0');
 	int x;
 	int len;
 
@@ -313,8 +380,8 @@ strlen_utf8(unsigned char **str)
 
 /* Start from @current and move back to @pos char. This pointer return. The
  * most left pointer is @start. */
-unsigned char *
-utf8_prevchar(unsigned char *current, int pos, unsigned char *start)
+char *
+utf8_prevchar(char *current, int pos, char *start)
 {
 	if (current == NULL || start == NULL || pos < 0)
 		return NULL;
@@ -329,12 +396,13 @@ utf8_prevchar(unsigned char *current, int pos, unsigned char *start)
 /* Count number of standard terminal cells needed for displaying UTF-8
  * character. */
 int
-utf8_char2cells(unsigned char *utf8_char, unsigned char *end)
+utf8_char2cells(const char *utf8_char_const, char *end)
 {
 	unicode_val_T u;
+	char *utf8_char = (char *)utf8_char_const;
 
 	if (end == NULL)
-		end = strchr((const char *)utf8_char, '\0');
+		end = strchr(utf8_char, '\0');
 
 	if(!utf8_char || !end)
 		return -1;
@@ -347,12 +415,12 @@ utf8_char2cells(unsigned char *utf8_char, unsigned char *end)
 /* Count number of standard terminal cells needed for displaying string
  * with UTF-8 characters. */
 int
-utf8_ptr2cells(unsigned char *string, unsigned char *end)
+utf8_ptr2cells(const char *string, char *end)
 {
 	int charlen, cell, cells = 0;
 
 	if (end == NULL)
-		end = strchr((const char *)string, '\0');
+		end = (char *)strchr(string, '\0');
 
 	if(!string || !end)
 		return -1;
@@ -375,12 +443,12 @@ utf8_ptr2cells(unsigned char *string, unsigned char *end)
 
 /* Count number of characters in string. */
 int
-utf8_ptr2chars(unsigned char *string, unsigned char *end)
+utf8_ptr2chars(char *string, char *end)
 {
 	int charlen, chars = 0;
 
 	if (end == NULL)
-		end = strchr((const char *)string, '\0');
+		end = strchr(string, '\0');
 
 	if(!string || !end)
 		return -1;
@@ -402,14 +470,14 @@ utf8_ptr2chars(unsigned char *string, unsigned char *end)
  * specified number of cells.
  */
 int
-utf8_cells2bytes(unsigned char *string, int max_cells, unsigned char *end)
+utf8_cells2bytes(const char *string, int max_cells, char *end)
 {
 	unsigned int bytes = 0, cells = 0;
 
 	assert(max_cells>=0);
 
 	if (end == NULL)
-		end = strchr((const char *)string, '\0');
+		end = (char *)strchr(string, '\0');
 
 	if(!string || !end)
 		return -1;
@@ -444,18 +512,18 @@ utf8_cells2bytes(unsigned char *string, int max_cells, unsigned char *end)
  *
  * This function can do some of the same jobs as utf8charlen(),
  * utf8_cells2bytes(), and strlen_utf8().  */
-unsigned char *
-utf8_step_forward(unsigned char *string, unsigned char *end,
+char *
+utf8_step_forward(char *string, char *end,
 		  int max, enum utf8_step way, int *count)
 {
 	int steps = 0;
-	unsigned char *current = string;
+	char *current = string;
 
 	assert(string);
 	assert(max >= 0);
 	if_assert_failed goto invalid_arg;
 	if (end == NULL)
-		end = strchr((const char *)string, '\0');
+		end = strchr(string, '\0');
 
 	switch (way) {
 	case UTF8_STEP_CHARACTERS:
@@ -470,7 +538,7 @@ utf8_step_forward(unsigned char *string, unsigned char *end,
 	case UTF8_STEP_CELLS_MORE:
 		while (steps < max && current < end) {
 			unicode_val_T u;
-			unsigned char *prev = current;
+			char *prev = current;
 			int width;
 
 			u = utf8_to_unicode(&current, end);
@@ -512,12 +580,12 @@ invalid_arg:
  * may be inconsistent.
  *
  * This function can do some of the same jobs as utf8_prevchar().  */
-unsigned char *
-utf8_step_backward(unsigned char *string, unsigned char *start,
+char *
+utf8_step_backward(char *string, char *start,
 		   int max, enum utf8_step way, int *count)
 {
 	int steps = 0;
-	unsigned char *current = string;
+	char *current = string;
 
 	assert(string);
 	assert(start);
@@ -536,8 +604,8 @@ utf8_step_backward(unsigned char *string, unsigned char *start,
 	case UTF8_STEP_CELLS_FEWER:
 	case UTF8_STEP_CELLS_MORE:
 		while (steps < max) {
-			unsigned char *prev = current;
-			unsigned char *look;
+			char *prev = current;
+			char *look;
 			unicode_val_T u;
 			int width;
 
@@ -605,12 +673,22 @@ invalid_arg:
  * libc version of wcwidth, and instead use a hardcoded mapping.
  *
  * @return	2 for double-width glyph, 1 for others.
- * 		TODO: May be extended to return 0 for zero-width glyphs
- * 		(like composing, maybe unprintable too).
+ * 		0 for unprintable glyphs (like 0x200e: "LEFT-TO-RIGHT MARK")
  */
+
+#if 0
 NONSTATIC_INLINE int
 unicode_to_cell(unicode_val_T c)
 {
+	int s;
+
+	if (!is_codepoint_supported(c)) {
+		BIN_SEARCH(unicode_7b, x, N_UNICODE_7B, c, s);
+		if (s != -1) return strlen(unicode_7b[s].s);
+	}
+
+	if (c == 0x200e || c == 0x200f)
+		return 0;
 	if (c >= 0x1100
 		&& (c <= 0x115f			/* Hangul Jamo */
 		|| c == 0x2329
@@ -629,6 +707,7 @@ unicode_to_cell(unicode_val_T c)
 
 	return 1;
 }
+#endif
 
 /* Fold the case of a Unicode character, so that hotkeys in labels can
  * be compared case-insensitively.  It is unspecified whether the
@@ -652,15 +731,15 @@ unicode_fold_label_case(unicode_val_T c)
 #endif /* CONFIG_UTF8 */
 
 NONSTATIC_INLINE unicode_val_T
-utf8_to_unicode(unsigned char **string, const unsigned char *end)
+utf8_to_unicode(char **string, const char *end)
 {
-	unsigned char *str = *string;
+	unsigned char *str = (unsigned char *)*string;
 	unicode_val_T u;
 	int length;
 
 	length = utf8char_len_tab[str[0]];
 
-	if (str + length > end) {
+	if (str + length > (const unsigned char *)end) {
 		return UCS_NO_CHAR;
 	}
 
@@ -731,7 +810,7 @@ invalid_utf8:
 			INTERNAL("utf8char_len_tab out of range");
 			goto invalid_utf8;
 	}
-	*string = str + length;
+	*string = (char *)(str + length);
 	return u;
 }
 
@@ -761,7 +840,7 @@ cp2u(int from, unsigned char c)
 }
 
 /* This slow and ugly code is used by the terminal utf_8_io */
-const unsigned char *
+const char *
 cp2utf8(int from, int c)
 {
 	from &= ~SYSTEM_CHARSET_FLAG;
@@ -773,7 +852,7 @@ cp2utf8(int from, int c)
 }
 
 unicode_val_T
-cp_to_unicode(int codepage, unsigned char **string, const unsigned char *end)
+cp_to_unicode(int codepage, char **string, const char *end)
 {
 	unicode_val_T ret;
 
@@ -806,12 +885,12 @@ get_combined(unicode_val_T *data, int length)
 
 	if (!combined_hash) combined_hash = init_hash8();
 	if (!combined_hash) return UCS_NO_CHAR;
-	item = get_hash_item(combined_hash, (unsigned char *)data, length * sizeof(*data));
+	item = get_hash_item(combined_hash, (char *)data, length * sizeof(*data));
 
-	if (item) return (unicode_val_T)(long)item->value;
+	if (item) return (unicode_val_T)(intptr_t)item->value;
 	if (last_combined >= UCS_END_COMBINED) return UCS_NO_CHAR;
 
-	key = mem_alloc((length + 1) * sizeof(*key));
+	key = (unicode_val_T *)mem_alloc((length + 1) * sizeof(*key));
 	if (!key) return UCS_NO_CHAR;
 	for (i = 0; i < length; i++)
 		key[i] = data[i];
@@ -820,15 +899,15 @@ get_combined(unicode_val_T *data, int length)
 	last_combined++;
 	indeks = last_combined - UCS_BEGIN_COMBINED;
 
-	combined = mem_realloc(combined, sizeof(*combined) * (indeks + 1));
+	combined = (unicode_val_T **)mem_realloc(combined, sizeof(*combined) * (indeks + 1));
 	if (!combined) {
 		mem_free(key);
 		last_combined--;
 		return UCS_NO_CHAR;
 	}
 	combined[indeks] = key;
-	item = add_hash_item(combined_hash, (unsigned char *)key,
-			     length * sizeof(*data), (void *)(long)(last_combined));
+	item = add_hash_item(combined_hash, (char *)key,
+			     length * sizeof(*data), (void *)(intptr_t)(last_combined));
 	if (!item) {
 		last_combined--;
 		mem_free(key);
@@ -852,19 +931,16 @@ free_combined()
 
 
 static void
-add_utf8(struct conv_table *ct, unicode_val_T u, const unsigned char *str)
+add_utf8(struct conv_table *ct, unicode_val_T u, const char *str)
 {
-	unsigned char *p = encode_utf8(u);
+	unsigned char *p = (unsigned char *)encode_utf8(u);
 
 	while (p[1]) {
 		if (ct[*p].t) ct = ct[*p].u.tbl;
 		else {
 			struct conv_table *nct;
 
-			assertm(ct[*p].u.str == no_str, "bad utf encoding #1");
-			if_assert_failed return;
-
-			nct = mem_calloc(256, sizeof(*nct));
+			nct = (struct conv_table *)mem_calloc(256, sizeof(*nct));
 			if (!nct) return;
 			new_translation_table(nct);
 			ct[*p].t = 1;
@@ -874,9 +950,6 @@ add_utf8(struct conv_table *ct, unicode_val_T u, const unsigned char *str)
 		p++;
 	}
 
-	assertm(!ct[*p].t, "bad utf encoding #2");
-	if_assert_failed return;
-
 	if (ct[*p].u.str == no_str)
 		ct[*p].u.str = str;
 }
@@ -895,7 +968,7 @@ free_utf_table(void)
 
 	/* Cast away const.  */
 	for (i = 128; i < 256; i++)
-		mem_free((unsigned char *) utf_table[i].u.str);
+		mem_free((char *) utf_table[i].u.str);
 }
 
 static struct conv_table *
@@ -1030,7 +1103,7 @@ get_translation_table(int from, int to)
 
 		for (i = 128; i < 256; i++) {
 			if (codepages[from].highhalf[i - 0x80] != 0xFFFF) {
-				const unsigned char *u;
+				const char *u;
 
 				u = u2cp(codepages[from].highhalf[i - 0x80], to);
 				if (u) table[i].u.str = u;
@@ -1042,7 +1115,7 @@ get_translation_table(int from, int to)
 }
 
 static inline int
-xxstrcmp(unsigned char *s1, unsigned char *s2, int l2)
+xxstrcmp(const char *s1, const char *s2, int l2)
 {
 	while (l2) {
 		if (*s1 > *s2) return 1;
@@ -1066,15 +1139,15 @@ struct entity_cache {
 	unsigned int hits;
 	int strlen;
 	int encoding;
-	const unsigned char *result;
-	unsigned char str[20]; /* Suffice in any case. */
+	const char *result;
+	char str[20]; /* Suffice in any case. */
 };
 
 /* comparison function for qsort() */
 static int
 hits_cmp(const void *v1, const void *v2)
 {
-	const struct entity_cache *a = v1, *b = v2;
+	const struct entity_cache *a = (const struct entity_cache *)v1, *b = (const struct entity_cache *)v2;
 
 	if (a->hits == b->hits) return 0;
 	if (a->hits > b->hits) return -1;
@@ -1087,14 +1160,14 @@ compare_entities(const void *key_, const void *element_)
 	struct string *key = (struct string *) key_;
 	struct entity *element = (struct entity *) element_;
 	int length = key->length;
-	unsigned char *first = key->source;
-	unsigned char *second = element->s;
+	const char *first = key->source;
+	const char *second = element->s;
 
 	return xxstrcmp(first, second, length);
 }
 
-const unsigned char *
-get_entity_string(const unsigned char *str, const int strlen, int encoding)
+const char *
+get_entity_string(const char *str, const int strlen, int encoding)
 {
 #define ENTITY_CACHE_SIZE 10	/* 10 seems a good value. */
 #define ENTITY_CACHE_MAXLEN 9   /* entities with length >= ENTITY_CACHE_MAXLEN or == 1
@@ -1102,7 +1175,7 @@ get_entity_string(const unsigned char *str, const int strlen, int encoding)
 	static struct entity_cache entity_cache[ENTITY_CACHE_MAXLEN][ENTITY_CACHE_SIZE];
 	static unsigned int nb_entity_cache[ENTITY_CACHE_MAXLEN];
 	unsigned int slen = 0;
-	const unsigned char *result = NULL;
+	const char *result = NULL;
 
 	/* Note that an object of static storage duration is automatically
 	 * initialised to zero in C.  */
@@ -1169,7 +1242,7 @@ skip:
 #endif /* CONFIG_UTF8 */
 	if (*str == '#') { /* Numeric entity. */
 		int l = (int) strlen;
-		unsigned char *st = (unsigned char *) str;
+		char *st = (char *) str;
 		unicode_val_T n = 0;
 
 		if (l == 1) goto end; /* &#; ? */
@@ -1209,8 +1282,8 @@ skip:
 		fprintf(stderr, "%lu %016x %s\n", (unsigned long) n , n, result);
 #endif
 	} else { /* Text entity. */
-		struct string key = INIT_STRING((unsigned char *) str, strlen);
-		struct entity *element = bsearch((void *) &key, entities,
+		struct string key = INIT_STRING((char *) str, strlen);
+		struct entity *element = (struct entity *)bsearch((void *) &key, entities,
 						 N_ENTITIES,
 						 sizeof(*element),
 						 compare_entities);
@@ -1266,17 +1339,17 @@ end:
 	return result;
 }
 
-unsigned char *
+char *
 convert_string(struct conv_table *convert_table,
-	       unsigned char *chars2, int charslen2, int cp,
+	       const char *chars2, int charslen2, int cp,
 	       enum convert_string_mode mode, int *length,
-	       void (*callback)(void *data, unsigned char *buf, int buflen),
+	       void (*callback)(void *data, char *buf, int buflen),
 	       void *callback_data)
 {
-	unsigned char *buffer;
+	char *buffer;
 	int bufferpos = 0;
 	int charspos = 0;
-	unsigned char *chars = chars2;
+	unsigned char *chars = (unsigned char *)chars2;
 	int charslen = charslen2;
 
 #ifdef HAVE_ICONV
@@ -1290,12 +1363,12 @@ convert_string(struct conv_table *convert_table,
 	int is_iconv = 0;
 	int chars_offset = 0;
 
-	if (!convert_table && !memchr(chars, '&', charslen)) {
+	if (!convert_table && !memchr((char *)chars, '&', charslen)) {
 		if (callback) {
-			if (charslen) callback(callback_data, chars, charslen);
+			if (charslen) callback(callback_data, (char *)chars, charslen);
 			return NULL;
 		} else {
-			return memacpy(chars, charslen);
+			return memacpy((char *)chars, charslen);
 		}
 	}
 
@@ -1311,7 +1384,7 @@ convert_string(struct conv_table *convert_table,
 
 	/* Buffer allocation */
 
-	buffer = mem_alloc(ALLOC_GR + 1 /* trailing \0 */);
+	buffer = (char *)mem_alloc(ALLOC_GR + 1 /* trailing \0 */);
 	if (!buffer) return NULL;
 
 #ifdef HAVE_ICONV
@@ -1320,7 +1393,7 @@ convert_string(struct conv_table *convert_table,
 		size_t before, to_copy;
 		char *outp, *inp;
 
-		if (iconv_cd >= 0) {
+		if (iconv_cd >= (iconv_t)0) {
 			if (cp != iconv_cp) {
 				iconv_close(iconv_cd);
 				iconv_cd = (iconv_t)-1;
@@ -1376,7 +1449,7 @@ repeat:
 
 out:
 	while (charspos < charslen) {
-		const unsigned char *translit;
+		const char *translit;
 
 #define PUTC do { \
 		buffer[bufferpos++] = chars[charspos++]; \
@@ -1393,12 +1466,12 @@ out:
 			t = convert_table;
 			i = charspos;
 
-			while (t[chars[i]].t) {
-				t = t[chars[i++]].u.tbl;
+			while (t[(unsigned char)chars[i]].t) {
+				t = t[(unsigned char)chars[i++]].u.tbl;
 				if (i >= charslen) PUTC;
 			}
 
-			translit = t[chars[i]].u.str;
+			translit = t[(unsigned char)chars[i]].u.str;
 			charspos = i + 1;
 
 		} else if (mode == CSM_FORM || mode == CSM_NONE) {
@@ -1421,7 +1494,7 @@ out:
 			if ((mode == CSM_DEFAULT || (chars[i] != '&' && chars[i] != '='))
 			    && i > start
 			    && !isasciialpha(chars[i]) && !isdigit(chars[i])) {
-				translit = get_entity_string(&chars[start], i - start,
+				translit = get_entity_string((const char *)&chars[start], i - start,
 						      cp);
 				if (chars[i] != ';') {
 					/* Eat &nbsp &nbsp<foo> happily, but
@@ -1445,7 +1518,7 @@ out:
 		}
 
 		while (*translit) {
-			unsigned char *new_;
+			char *new_;
 
 			buffer[bufferpos++] = *(translit++);
 flush:
@@ -1456,7 +1529,7 @@ flush:
 				callback(callback_data, buffer, bufferpos);
 				bufferpos = 0;
 			} else {
-				new_ = mem_realloc(buffer, bufferpos + ALLOC_GR);
+				new_ = (char *)mem_realloc(buffer, bufferpos + ALLOC_GR);
 				if (!new_) {
 					mem_free(buffer);
 					return NULL;
@@ -1487,7 +1560,7 @@ flush:
 
 #ifndef USE_FASTFIND
 int
-get_cp_index(const unsigned char *name)
+get_cp_index(const char *name)
 {
 	int i, a;
 	int syscp = 0;
@@ -1496,8 +1569,17 @@ get_cp_index(const unsigned char *name)
 #if HAVE_LANGINFO_CODESET
 		name = nl_langinfo(CODESET);
 		syscp = SYSTEM_CHARSET_FLAG;
+#else
+#ifdef CONFIG_OS_DOS
+		int cp = os_default_charset();
+		if (cp != -1) {
+			return cp | SYSTEM_CHARSET_FLAG;
+		} else {
+			name = "us-ascii";
+		}
 #else
 		name = "us-ascii";
+#endif
 #endif
 	}
 
@@ -1568,7 +1650,7 @@ static struct fastfind_index ff_charsets_index
 /* It searchs for a charset named @name or one of its aliases and
  * returns index for it or -1 if not found. */
 int
-get_cp_index(const unsigned char *name)
+get_cp_index(const char *name)
 {
 	const struct codepage_desc *codepage;
 	int syscp = 0;
@@ -1577,12 +1659,21 @@ get_cp_index(const unsigned char *name)
 #if HAVE_LANGINFO_CODESET
 		name = nl_langinfo(CODESET);
 		syscp = SYSTEM_CHARSET_FLAG;
+#else
+#ifdef CONFIG_OS_DOS
+		int cp = os_default_charset();
+		if (cp != -1) {
+			return cp | SYSTEM_CHARSET_FLAG;
+		} else {
+			name = "us-ascii";
+		}
 #else
 		name = "us-ascii";
+#endif
 #endif
 	}
 
-	codepage = fastfind_search(&ff_charsets_index, name, strlen(name));
+	codepage = (const struct codepage_desc *)fastfind_search(&ff_charsets_index, name, strlen(name));
 	if (codepage) {
 		assert(codepages <= codepage && codepage < codepages + N_CODEPAGES);
 		return (codepage - codepages) | syscp;
@@ -1597,9 +1688,78 @@ get_cp_index(const unsigned char *name)
 
 #endif /* USE_FASTFIND */
 
+/* create the list of codepoints supported by the terminal */
+
+#if defined(GIO_UNIMAP) && defined(CONFIG_CODEPOINT)
+static int
+cmpint(const void *a, const void *b)
+{
+	if (* (int *) a < * (int *) b)
+		return -1;
+	else if (* (int *) a == * (int *) b)
+		return 0;
+	else
+		return 1;
+}
+
+void
+make_codepoints(void)
+{
+	int tty;
+	struct unimapdesc table;
+	int res;
+	int i;
+
+	tty = get_ctl_handle();
+	if (tty == -1) {
+		codepoints.size = -1;
+		return ;
+	}
+
+	table.entry_ct = 0;
+	table.entries = NULL;
+	res = ioctl(tty, GIO_UNIMAP, &table);
+	if (res && errno != ENOMEM) {
+#ifdef CONFIG_DEBUG
+		perror("GIO_UNIMAP");
+#endif
+		codepoints.size = -1;
+		return;
+	}
+
+	table.entries = (struct unipair *)malloc(table.entry_ct * sizeof(struct unipair));
+	res = ioctl(tty, GIO_UNIMAP, &table);
+	if (res) {
+#ifdef CONFIG_DEBUG
+		perror("GIO_UNIMAP");
+#endif
+		close(tty);
+		codepoints.size = -1;
+		return;
+	}
+
+	codepoints.size = table.entry_ct;
+	codepoints.list = (unicode_val_T *)malloc(table.entry_ct * sizeof(unicode_val_T));
+	for (i = 0; i < table.entry_ct; i++)
+		codepoints.list[i] = table.entries[i].unicode;
+
+	qsort(codepoints.list, codepoints.size, sizeof(unicode_val_T), cmpint);
+
+	// for (i = 0; i < codepoints.size; i++)
+	//	fprintf(stderr, "U+%04X\n", codepoints.list[i]);
+}
+#else
+void
+make_codepoints(void)
+{
+	codepoints.size = -1;
+}
+#endif
+
 void
 init_charsets_lookup(void)
 {
+	make_codepoints();
 #ifdef USE_FASTFIND
 	fastfind_index(&ff_charsets_index, FF_COMPRESS);
 #endif
@@ -1618,7 +1778,7 @@ free_charsets_lookup(void)
  * localize these with gettext.  So it may be best not to use this
  * function if the name will have to be converted back to an
  * index.  */
-unsigned char *
+const char *
 get_cp_name(int cp_index)
 {
 	if (cp_index < 0) return "none";
@@ -1630,20 +1790,20 @@ get_cp_name(int cp_index)
 /* Get the codepage's name for saving to a configuration file.  These
  * names can be converted back to indexes, even in future versions of
  * ELinks.  */
-unsigned char *
+const char *
 get_cp_config_name(int cp_index)
 {
 	if (cp_index < 0) return "none";
 	if (cp_index & SYSTEM_CHARSET_FLAG) return "System";
 	if (!codepages[cp_index].aliases) return NULL;
 
-	return codepages[cp_index].aliases[0];
+	return (char *)codepages[cp_index].aliases[0];
 }
 
 /* Get the codepage's name for sending to a library or server that
  * understands MIME charset names.  This function irreversibly maps
  * the "System" codepage to the underlying charset.  */
-unsigned char *
+const char *
 get_cp_mime_name(int cp_index)
 {
 	if (cp_index < 0) return "none";
@@ -1662,7 +1822,7 @@ is_cp_utf8(int cp_index)
 
 /* This function will be used by the xhtml parser. */
 const uint16_t *
-get_cp_highhalf(const unsigned char *name)
+get_cp_highhalf(const char *name)
 {
 	int cp = get_cp_index(name);
 
