@@ -18,6 +18,10 @@
 #error "Huh?! You have SSL enabled, but not OPENSSL nor GNUTLS!! And then you want exactly *what* from me?"
 #endif
 
+#ifdef HAVE_WS2TCPIP_H
+#include <ws2tcpip.h> /* MinGW */
+#endif
+
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
@@ -56,14 +60,13 @@
 
 #ifdef USE_OPENSSL
 
-#define ssl_do_connect(socket)		SSL_get_error(socket->ssl, SSL_connect(socket->ssl))
-#define ssl_do_write(socket, data, len)	SSL_write(socket->ssl, data, len)
-#define ssl_do_read(socket, data, len)	SSL_read(socket->ssl, data, len)
+#define ssl_do_connect(socket)		SSL_get_error((SSL *)socket->ssl, SSL_connect((SSL *)socket->ssl))
+#define ssl_do_write(socket, data, len)	SSL_write((SSL *)socket->ssl, data, len)
+#define ssl_do_read(socket, data, len)	SSL_read((SSL *)socket->ssl, data, len)
 #define ssl_do_close(socket)		/* Hmh? No idea.. */
 
 #elif defined(CONFIG_GNUTLS)
 
-#define ssl_do_connect(conn)		gnutls_handshake(*((ssl_t *) socket->ssl))
 #define ssl_do_write(socket, data, len)	gnutls_record_send(*((ssl_t *) socket->ssl), data, len)
 #define ssl_do_read(socket, data, len)	gnutls_record_recv(*((ssl_t *) socket->ssl), data, len)
 /* We probably don't handle this entirely correctly.. */
@@ -71,6 +74,15 @@
 
 #endif
 
+#ifdef WIN32
+#define SOCK_SHIFT 1024
+#endif
+
+/* Definition of X509_NAME causes compilation error on WIN32
+ * due to X509_NAME redefinition in wincrypt.h */
+#ifdef WIN32
+#undef X509_NAME
+#endif
 
 /* Refuse to negotiate TLS 1.0 and later protocols on @socket->ssl.
  * Without this, connecting to <https://www-s.uiuc.edu/> with GnuTLS
@@ -81,7 +93,9 @@ static void
 ssl_set_no_tls(struct socket *socket)
 {
 #ifdef CONFIG_OPENSSL
-	((ssl_t *) socket->ssl)->options |= SSL_OP_NO_TLSv1;
+#ifdef SSL_OP_NO_TLSv1
+	SSL_set_options((ssl_t *)socket->ssl, SSL_OP_NO_TLSv1);
+#endif
 #elif defined(CONFIG_GNUTLS)
 	/* There is another gnutls_priority_set_direct call elsewhere
 	 * in ELinks.  If you change the priorities here, please check
@@ -105,9 +119,9 @@ verify_certificates(struct socket *socket)
 {
 	gnutls_x509_crt_t cert;
 	gnutls_session_t session = *(ssl_t *)socket->ssl;
-	struct connection *conn = socket->conn;
+	struct connection *conn = (struct connection *)socket->conn;
 	const gnutls_datum_t *cert_list;
-	unsigned char *hostname;
+	char *hostname;
 	int ret;
 	unsigned int cert_list_size, status;
 
@@ -156,7 +170,7 @@ verify_certificates(struct socket *socket)
 	 * only contain ASCII characters.  Internationalized domain
 	 * names must thus be in Punycode form.  Because GnuTLS 2.8.6
 	 * does not itself support IDN, ELinks must convert.  */
-	hostname = get_uri_string(conn->proxied_uri, URI_HOST | URI_IDN);
+	hostname = get_uri_string(conn->proxied_uri, URI_DNS_HOST);
 	if (!hostname) return -6;
 
 	ret = !gnutls_x509_crt_check_hostname(cert, hostname);
@@ -190,7 +204,7 @@ verify_certificates(struct socket *socket)
  * SubjectAltName, rather than in commonName.  For comparing those,
  * match_uri_host_ip() must be used instead of this function.  */
 static int
-match_uri_host_name(const unsigned char *uri_host,
+match_uri_host_name(const char *uri_host,
 		    ASN1_STRING *cert_host_asn1)
 {
 	const size_t uri_host_len = strlen(uri_host);
@@ -210,7 +224,7 @@ match_uri_host_name(const unsigned char *uri_host,
 		goto mismatch;
 
 	matched = match_hostname_pattern(uri_host, uri_host_len,
-					 cert_host, cert_host_len);
+					 (char *)cert_host, cert_host_len);
 
 mismatch:
 	if (cert_host)
@@ -243,10 +257,15 @@ mismatch:
  * in the name constraints extension of a CA certificate (RFC 5280
  * section 4.2.1.10).  */
 static int
-match_uri_host_ip(const unsigned char *uri_host,
+match_uri_host_ip(const char *uri_host,
 		  ASN1_OCTET_STRING *cert_host_asn1)
 {
+#ifdef HAVE_ASN1_STRING_GET0_DATA
+	const unsigned char *cert_host_addr = ASN1_STRING_get0_data(cert_host_asn1);
+#else
 	const unsigned char *cert_host_addr = ASN1_STRING_data(cert_host_asn1);
+#endif
+
 	struct in_addr uri_host_in;
 #ifdef CONFIG_IPV6
 	struct in6_addr uri_host_in6;
@@ -273,13 +292,20 @@ match_uri_host_ip(const unsigned char *uri_host,
 	 * network byte order.  */
 	switch (ASN1_STRING_length(cert_host_asn1)) {
 	case 4:
+#if defined(HAVE_INET_PTON)
+		return inet_pton(AF_INET, uri_host, &uri_host_in) != 0
+		    && memcmp(cert_host_addr, &uri_host_in.s_addr, 4) == 0;
+#elif defined(HAVE_INET_ATON)
 		return inet_aton(uri_host, &uri_host_in) != 0
 		    && memcmp(cert_host_addr, &uri_host_in.s_addr, 4) == 0;
+#endif
 
 #ifdef CONFIG_IPV6
 	case 16:
+#ifdef HAVE_INET_PTON
 		return inet_pton(AF_INET6, uri_host, &uri_host_in6) == 1
 		    && memcmp(cert_host_addr, &uri_host_in6.s6_addr, 16) == 0;
+#endif
 #endif
 
 	default:
@@ -292,11 +318,12 @@ match_uri_host_ip(const unsigned char *uri_host,
 static int
 verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 {
+
 	X509 *cert;
 	SSL *ssl;
 	struct socket *socket;
 	struct connection *conn;
-	unsigned char *host_in_uri;
+	char *host_in_uri;
 	GENERAL_NAMES *alts;
 	int saw_dns_name = 0;
 	int matched = 0;
@@ -310,17 +337,17 @@ verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 		return preverify_ok;
 
 	cert = X509_STORE_CTX_get_current_cert(ctx);
-	ssl = X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
-	socket = SSL_get_ex_data(ssl, socket_SSL_ex_data_idx);
-	conn = socket->conn;
-	host_in_uri = get_uri_string(conn->proxied_uri, URI_HOST | URI_IDN);
+	ssl = (SSL *)X509_STORE_CTX_get_ex_data(ctx, SSL_get_ex_data_X509_STORE_CTX_idx());
+	socket = (struct socket *)SSL_get_ex_data(ssl, socket_SSL_ex_data_idx);
+	conn = (struct connection *)socket->conn;
+	host_in_uri = get_uri_string(conn->proxied_uri, URI_DNS_HOST);
 	if (!host_in_uri)
 		return 0;
 
 	/* RFC 5280 section 4.2.1.6 describes the subjectAltName extension.
 	 * RFC 2818 section 3.1 says Common Name must not be used
 	 * if dNSName is present.  */
-	alts = X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
+	alts = (GENERAL_NAMES *)X509_get_ext_d2i(cert, NID_subject_alt_name, NULL, NULL);
 	if (alts != NULL) {
 		int alt_count;
 		int alt_pos;
@@ -342,7 +369,6 @@ verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 		/* Free the GENERAL_NAMES list and each element.  */
 		sk_GENERAL_NAME_pop_free(alts, GENERAL_NAME_free);
 	}
-
 	if (!matched && !saw_dns_name) {
 		X509_NAME *name;
 		int cn_index;
@@ -363,6 +389,22 @@ verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
 
 #endif	/* USE_OPENSSL */
 
+#if defined(CONFIG_GNUTLS)
+static int
+ssl_do_connect(struct socket *socket)
+{
+	int ret;
+
+	gnutls_handshake_set_timeout(*(ssl_t *)(socket->ssl), GNUTLS_DEFAULT_HANDSHAKE_TIMEOUT);
+
+	do {
+		ret = gnutls_handshake(*(ssl_t *)socket->ssl);
+	} while (ret < 0 && !gnutls_error_is_fatal(ret));
+
+	return ret;
+}
+#endif
+
 static void
 ssl_want_read(struct socket *socket)
 {
@@ -372,7 +414,7 @@ ssl_want_read(struct socket *socket)
 	switch (ssl_do_connect(socket)) {
 		case SSL_ERROR_NONE:
 #ifdef CONFIG_GNUTLS
-			if (get_opt_bool("connection.ssl.cert_verify", NULL)
+			if (socket->verify && get_opt_bool("connection.ssl.cert_verify", NULL)
 			    && verify_certificates(socket)) {
 				socket->ops->retry(socket, connection_state(S_SSL_ERROR));
 				return;
@@ -398,11 +440,17 @@ int
 ssl_connect(struct socket *socket)
 {
 	int ret;
-	unsigned char *server_name;
-	struct connection *conn = socket->conn;
+	char *server_name;
+	struct connection *conn = (struct connection *)socket->conn;
+#ifdef WIN32
+	int ssl_sock_fd = socket->fd - SOCK_SHIFT;
+#endif
+
+
 
 	/* TODO: Recode server_name to UTF-8.  */
 	server_name = get_uri_string(conn->proxied_uri, URI_HOST);
+
 	if (!server_name) {
 		socket->ops->done(socket, connection_state(S_OUT_OF_MEM));
 		return -1;
@@ -424,15 +472,33 @@ ssl_connect(struct socket *socket)
 		ssl_set_no_tls(socket);
 
 #ifdef USE_OPENSSL
-	SSL_set_fd(socket->ssl, socket->fd);
 
-	if (get_opt_bool("connection.ssl.cert_verify", NULL))
-		SSL_set_verify(socket->ssl, SSL_VERIFY_PEER
+#ifndef WIN32
+	SSL_set_fd((SSL *)socket->ssl, socket->fd);
+#else
+	SSL_set_fd((SSL *)socket->ssl, ssl_sock_fd);
+#endif
+
+	if (socket->verify) {
+		if (conn->proxied_uri->protocol == PROTOCOL_HTTPS) {
+			if (get_opt_bool("connection.ssl.cert_verify", NULL)) {
+				SSL_set_verify((SSL *)socket->ssl, SSL_VERIFY_PEER
 					  | SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
-			       verify_callback);
-
+					  verify_callback);
+			}
+		}
+#ifdef CONFIG_GEMINI
+		else if (conn->proxied_uri->protocol == PROTOCOL_GEMINI) {
+			if (get_opt_bool("connection.ssl.gemini_cert_verify", NULL)) {
+				SSL_set_verify((SSL *)socket->ssl, SSL_VERIFY_PEER
+					  | SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
+					  verify_callback);
+			}
+		}
+#endif
+	}
 	if (get_opt_bool("connection.ssl.client_cert.enable", NULL)) {
-		unsigned char *client_cert;
+		char *client_cert;
 
 #ifdef CONFIG_NSS_COMPAT_OSSL
 		client_cert = get_opt_str(
@@ -453,7 +519,7 @@ ssl_connect(struct socket *socket)
 					(SSL *) socket->ssl,
 					client_cert);
 #else
-			SSL_CTX *ctx = ((SSL *) socket->ssl)->ctx;
+			SSL_CTX *ctx = SSL_get_SSL_CTX((SSL *) socket->ssl);
 
 			SSL_CTX_use_certificate_chain_file(ctx, client_cert);
 			SSL_CTX_use_PrivateKey_file(ctx, client_cert,
@@ -488,7 +554,7 @@ ssl_connect(struct socket *socket)
 
 		case SSL_ERROR_NONE:
 #ifdef CONFIG_GNUTLS
-			if (!get_opt_bool("connection.ssl.cert_verify", NULL))
+			if (!socket->verify || !get_opt_bool("connection.ssl.cert_verify", NULL))
 				break;
 
 			if (!verify_certificates(socket))
@@ -500,7 +566,6 @@ ssl_connect(struct socket *socket)
 				/* DBG("sslerr %s", gnutls_strerror(ret)); */
 				socket->no_tls = !socket->no_tls;
 			}
-
 			connect_socket(socket, connection_state(S_SSL_ERROR));
 			return -1;
 	}
@@ -510,13 +575,13 @@ ssl_connect(struct socket *socket)
 
 /* Return enum socket_error on error, bytes written on success. */
 ssize_t
-ssl_write(struct socket *socket, unsigned char *data, int len)
+ssl_write(struct socket *socket, char *data, int len)
 {
 	ssize_t wr = ssl_do_write(socket, data, len);
 
 	if (wr <= 0) {
 #ifdef USE_OPENSSL
-		int err = SSL_get_error(socket->ssl, wr);
+		int err = SSL_get_error((SSL *)socket->ssl, wr);
 #elif defined(CONFIG_GNUTLS)
 		int err = wr;
 #endif
@@ -539,20 +604,28 @@ ssl_write(struct socket *socket, unsigned char *data, int len)
 
 /* Return enum socket_error on error, bytes read on success. */
 ssize_t
-ssl_read(struct socket *socket, unsigned char *data, int len)
+ssl_read(struct socket *socket, char *data, int len)
 {
 	ssize_t rd = ssl_do_read(socket, data, len);
 
 	if (rd <= 0) {
 #ifdef USE_OPENSSL
-		int err = SSL_get_error(socket->ssl, rd);
+		int err = SSL_get_error((SSL *)socket->ssl, rd);
 #elif defined(CONFIG_GNUTLS)
 		int err = rd;
 #endif
 
 #ifdef CONFIG_GNUTLS
-		if (err == GNUTLS_E_REHANDSHAKE)
-			return -1;
+		if (err == GNUTLS_E_PREMATURE_TERMINATION) {
+			return SOCKET_CANT_READ;
+		}
+
+		if (err == GNUTLS_E_REHANDSHAKE) {
+			err = ssl_do_connect(socket);
+			if (err == 0) {
+				return SOCKET_SSL_WANT_READ;
+			}
+		}
 #endif
 
 		if (err == SSL_ERROR_WANT_READ ||
