@@ -8,9 +8,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_LIMITS_H
-#include <limits.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h> /* OS/2 needs this after sys/types.h */
 #endif
+#include <limits.h>
 
 #include "elinks.h"
 
@@ -18,7 +20,7 @@
 #include "config/options.h"
 #include "cookies/cookies.h"
 #include "intl/charsets.h"
-#include "intl/gettext/libintl.h"
+#include "intl/libintl.h"
 #include "main/module.h"
 #include "network/connection.h"
 #include "network/progress.h"
@@ -65,64 +67,64 @@
 
 static struct auth_entry proxy_auth;
 
-static unsigned char *accept_charset = NULL;
+static char *accept_charset = NULL;
 
 
 static union option_info http_options[] = {
 	INIT_OPT_TREE("protocol", N_("HTTP"),
-		"http", 0,
+		"http", OPT_ZERO,
 		N_("HTTP-specific options.")),
 
 
 	INIT_OPT_TREE("protocol.http", N_("Server bug workarounds"),
-		"bugs", 0,
+		"bugs", OPT_ZERO,
 		N_("Server-side HTTP bugs workarounds.")),
 
 	INIT_OPT_BOOL("protocol.http.bugs", N_("Do not send Accept-Charset"),
-		"accept_charset", 0, 1,
+		"accept_charset", OPT_ZERO, 1,
 		N_("The Accept-Charset header is quite long and sending it "
 		"can trigger bugs in some rarely found servers.")),
 
 	INIT_OPT_BOOL("protocol.http.bugs", N_("Allow blacklisting"),
-		"allow_blacklist", 0, 1,
+		"allow_blacklist", OPT_ZERO, 1,
 		N_("Allow blacklisting of buggy servers.")),
 
 	INIT_OPT_BOOL("protocol.http.bugs", N_("Broken 302 redirects"),
-		"broken_302_redirect", 0, 1,
+		"broken_302_redirect", OPT_ZERO, 1,
 		N_("Broken 302 redirect (violates RFC but compatible with "
 		"Netscape). This is a problem for a lot of web discussion "
 		"boards and the like. If they will do strange things to you, "
 		"try to play with this.")),
 
 	INIT_OPT_BOOL("protocol.http.bugs", N_("No keepalive after POST requests"),
-		"post_no_keepalive", 0, 0,
+		"post_no_keepalive", OPT_ZERO, 0,
 		N_("Disable keepalive connection after POST request.")),
 
 	INIT_OPT_BOOL("protocol.http.bugs", N_("Use HTTP/1.0"),
-		"http10", 0, 0,
+		"http10", OPT_ZERO, 0,
 		N_("Use HTTP/1.0 protocol instead of HTTP/1.1.")),
 
 	INIT_OPT_TREE("protocol.http", N_("Proxy configuration"),
-		"proxy", 0,
+		"proxy", OPT_ZERO,
 		N_("HTTP proxy configuration.")),
 
 	INIT_OPT_STRING("protocol.http.proxy", N_("Host and port-number"),
-		"host", 0, "",
+		"host", OPT_ZERO, "",
 		N_("Host and port-number (host:port) of the HTTP proxy, "
 		"or blank. If it's blank, HTTP_PROXY environment variable "
 		"is checked as well.")),
 
 	INIT_OPT_STRING("protocol.http.proxy", N_("Username"),
-		"user", 0, "",
+		"user", OPT_ZERO, "",
 		N_("Proxy authentication username.")),
 
 	INIT_OPT_STRING("protocol.http.proxy", N_("Password"),
-		"passwd", 0, "",
+		"passwd", OPT_ZERO, "",
 		N_("Proxy authentication password.")),
 
 
 	INIT_OPT_TREE("protocol.http", N_("Referer sending"),
-		"referer", 0,
+		"referer", OPT_ZERO,
 		N_("HTTP referer sending options. HTTP referer is a special "
 		"header sent in the HTTP requests, which is supposed to "
 		"contain the previous page visited by the browser."
@@ -132,7 +134,7 @@ static union option_info http_options[] = {
 		"to a security problem on some badly designed web pages.")),
 
 	INIT_OPT_INT("protocol.http.referer", N_("Policy"),
-		"policy", 0,
+		"policy", OPT_ZERO,
 		REFERER_NONE, REFERER_TRUE, REFERER_TRUE,
 		N_("Mode of sending HTTP referer:\n"
 		"0 is send no referer\n"
@@ -141,16 +143,19 @@ static union option_info http_options[] = {
 		"3 is send previous URL as referer (correct, but insecure)")),
 
 	INIT_OPT_STRING("protocol.http.referer", N_("Fake referer URL"),
-		"fake", 0, "",
+		"fake", OPT_ZERO, "",
 		N_("Fake referer to be sent when policy is 2.")),
 
+	INIT_OPT_STRING("protocol.http", N_("Accept header"),
+		"accept", OPT_ZERO, "text/html, text/*;q=0.9, */*;q=0.8",
+		N_("Value of the Accept header field.")),
 
 	INIT_OPT_STRING("protocol.http", N_("Send Accept-Language header"),
-		"accept_language", 0, "",
+		"accept_language", OPT_ZERO, "",
 		N_("Send Accept-Language header.")),
 
 	INIT_OPT_BOOL("protocol.http", N_("Use UI language as Accept-Language"),
-		"accept_ui_language", 0, 1,
+		"accept_ui_language", OPT_ZERO, 1,
 		N_("Request localised versions of documents from web-servers "
 		"(using the Accept-Language header) using the language "
 		"you have configured for ELinks' user-interface (this also "
@@ -169,7 +174,7 @@ static union option_info http_options[] = {
 	 * or perhaps even blacklist the server for compression and retry.
 	 * Until that has been implemented, disable compression by default.  */
 	INIT_OPT_BOOL("protocol.http", N_("Enable on-the-fly compression"),
-		"compression", 0, 0,
+		"compression", OPT_ZERO, 0,
 		N_("If enabled, the capability to receive compressed content "
 		"(gzip and/or bzip2) is announced to the server, which "
 		"usually sends the reply compressed, thus saving some "
@@ -184,7 +189,7 @@ static union option_info http_options[] = {
 		"see Help -> About.")),
 
 	INIT_OPT_BOOL("protocol.http", N_("Activate HTTP TRACE debugging"),
-		"trace", 0, 0,
+		"trace", OPT_ZERO, 0,
 		N_("If active, all HTTP requests are sent with TRACE as "
 		"their method rather than GET or POST. This is useful for "
 		"debugging of both ELinks and various server-side scripts "
@@ -194,7 +199,7 @@ static union option_info http_options[] = {
 
 	/* OSNews.com is supposed to be relying on the textmode token, at least. */
 	INIT_OPT_STRING("protocol.http", N_("User-agent identification"),
-		"user_agent", 0, "ELinks/%v (textmode; %s; %t-%b)",
+		"user_agent", OPT_ZERO, "ELinks/%v (textmode; %s; %t-%b)",
 		N_("Change the User Agent ID. That means identification "
 		"string, which is sent to HTTP server when a document is "
 		"requested. The 'textmode' token in the first field is our "
@@ -214,22 +219,22 @@ static union option_info http_options[] = {
 
 
 	INIT_OPT_TREE("protocol", N_("HTTPS"),
-  		"https", 0,
+		"https", OPT_ZERO,
 		N_("HTTPS-specific options.")),
 
 	INIT_OPT_TREE("protocol.https", N_("Proxy configuration"),
-	  	"proxy", 0,
+		"proxy", OPT_ZERO,
 		N_("HTTPS proxy configuration.")),
 
 	INIT_OPT_STRING("protocol.https.proxy", N_("Host and port-number"),
-	  	"host", 0, "",
+		"host", OPT_ZERO, "",
 		N_("Host and port-number (host:port) of the HTTPS CONNECT "
 		"proxy, or blank. If it's blank, HTTPS_PROXY environment "
 		"variable is checked as well.")),
 	NULL_OPTION_INFO,
 };
 
-static void done_http();
+static void done_http(struct module *mod);
 
 struct module http_protocol_module = struct_module(
 	/* name: */		N_("HTTP"),
@@ -243,7 +248,7 @@ struct module http_protocol_module = struct_module(
 
 
 static void
-done_http(void)
+done_http(struct module *mod)
 {
 	mem_free_if(proxy_auth.realm);
 	mem_free_if(proxy_auth.nonce);
@@ -259,7 +264,7 @@ static void
 init_accept_charset(void)
 {
 	struct string ac;
-	unsigned char *cs;
+	const char *cs;
 	int i;
 
 	if (!init_string(&ac)) return;
@@ -283,9 +288,9 @@ init_accept_charset(void)
 }
 
 
-unsigned char *
-subst_user_agent(unsigned char *fmt, unsigned char *version,
-		 unsigned char *sysname, unsigned char *termsize)
+char *
+subst_user_agent(char *fmt, const char *version,
+		 char *sysname, char *termsize)
 {
 	struct string agent;
 
@@ -305,9 +310,9 @@ subst_user_agent(unsigned char *fmt, unsigned char *version,
 		switch (*fmt) {
 			case 'b':
 				if (!list_empty(sessions)) {
-					unsigned char bs[4] = "";
+					char bs[4] = "";
 					int blen = 0;
-					struct session *ses = sessions.prev;
+					struct session *ses = (struct session *)sessions.prev;
 					int bars = ses->status.show_status_bar
 						+ ses->status.show_tabs_bar
 						+ ses->status.show_title_bar;
@@ -337,7 +342,7 @@ subst_user_agent(unsigned char *fmt, unsigned char *version,
 }
 
 static void
-add_url_to_http_string(struct string *header, struct uri *uri, int components)
+add_url_to_http_string(struct string *header, struct uri *uri, uri_component_T components)
 {
 	/* This block substitues spaces in URL by %20s. This is
 	 * certainly not the right place where to do it, but now the
@@ -345,10 +350,22 @@ add_url_to_http_string(struct string *header, struct uri *uri, int components)
 	 * before. We should probably encode all URLs as early as
 	 * possible, and possibly decode them back in protocol
 	 * backends. --pasky */
-	unsigned char *string = get_uri_string(uri, components);
-	unsigned char *data = string;
+	struct string encoded;
+	char *string;
+	char *data;
 
-	if (!string) return;
+	if (!init_string(&encoded)) {
+		return;
+	}
+	string = get_uri_string(uri, components);
+
+	if (!string) {
+		done_string(&encoded);
+		return;
+	}
+	encode_uri_string_percent(&encoded, string, -1);
+	mem_free(string);
+	data = encoded.source;
 
 	while (*data) {
 		int len = strcspn(data, " \t\r\n\\");
@@ -364,15 +381,14 @@ add_url_to_http_string(struct string *header, struct uri *uri, int components)
 
 		data	+= len;
 	}
-
-	mem_free(string);
+	done_string(&encoded);
 }
 
 /* Parse from @end - 1 to @start and set *@value to integer found.
  * It returns -1 if not a number, 0 otherwise.
  * @end should be > @start. */
 static int
-revstr2num(unsigned char *start, unsigned char *end, int *value)
+revstr2num(char *start, char *end, int *value)
 {
 	int q = 1, val = 0;
 
@@ -394,8 +410,8 @@ revstr2num(unsigned char *start, unsigned char *end, int *value)
 static int
 get_http_code(struct read_buffer *rb, int *code, struct http_version *version)
 {
-	unsigned char *head = rb->data;
-	unsigned char *start;
+	char *head = rb->data;
+	char *start;
 
 	*code = 0;
 	version->major = 0;
@@ -450,11 +466,11 @@ get_http_code(struct read_buffer *rb, int *code, struct http_version *version)
 
 static int
 check_http_server_bugs(struct uri *uri, struct http_connection_info *http,
-		       unsigned char *head)
+		       char *head)
 {
-	unsigned char *server;
-	const unsigned char *const *s;
-	static const unsigned char *const buggy_servers[] = {
+	char *server;
+	const char *const *s;
+	static const char *const buggy_servers[] = {
 		"mod_czech/3.1.0",
 		"Purveyor",
 		"Netscape-Enterprise",
@@ -470,7 +486,7 @@ check_http_server_bugs(struct uri *uri, struct http_connection_info *http,
 		return 0;
 
 	for (s = buggy_servers; *s; s++) {
-		if (strstr((const char *)server, *s)) {
+		if (strstr(server, *s)) {
 			add_blacklist_entry(uri, SERVER_BLACKLIST_HTTP10);
 			break;
 		}
@@ -490,7 +506,7 @@ http_end_request(struct connection *conn, struct connection_state state,
 
 	/* shutdown_connection_stream() should not change conn->info,
 	 * but in case it does, read conn->info only after the call.  */
-	http = conn->info;
+	http = (struct http_connection_info *)conn->info;
 	if (http)
 		done_http_post(&http->post);
 
@@ -538,7 +554,7 @@ proxy_protocol_handler(struct connection *conn)
 static void
 done_http_connection(struct connection *conn)
 {
-	struct http_connection_info *http = conn->info;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
 
 	done_http_post(&http->post);
 	mem_free(http);
@@ -551,7 +567,7 @@ init_http_connection_info(struct connection *conn, int major, int minor, int clo
 {
 	struct http_connection_info *http;
 
-	http = mem_calloc(1, sizeof(*http));
+	http = (struct http_connection_info *)mem_calloc(1, sizeof(*http));
 	if (!http) {
 		http_end_request(conn, connection_state(S_OUT_OF_MEM), 0);
 		return NULL;
@@ -588,12 +604,18 @@ init_http_connection_info(struct connection *conn, int major, int minor, int clo
 static void
 accept_encoding_header(struct string *header)
 {
-#if defined(CONFIG_GZIP) || defined(CONFIG_BZIP2) || defined(CONFIG_LZMA) || defined(CONFIG_BROTLI)
+#if defined(CONFIG_GZIP) || defined(CONFIG_BZIP2) || defined(CONFIG_LZMA) || defined(CONFIG_BROTLI) || defined(CONFIG_ZSTD)
 	int comma = 0;
 
 	add_to_string(header, "Accept-Encoding: ");
 
+#ifdef CONFIG_ZSTD
+	add_to_string(header, "zstd");
+	comma = 1;
+#endif
+
 #ifdef CONFIG_BROTLI
+	if (comma) add_to_string(header, ", ");
 	add_to_string(header, "br");
 	comma = 1;
 #endif
@@ -606,7 +628,7 @@ accept_encoding_header(struct string *header)
 
 #ifdef CONFIG_GZIP
 	if (comma) add_to_string(header, ", ");
-	add_to_string(header, "gzip, deflate");
+	add_to_string(header, "gzip");
 	comma = 1;
 #endif
 
@@ -618,14 +640,14 @@ accept_encoding_header(struct string *header)
 #endif
 }
 
-#define POST_BUFFER_SIZE 16384
+#define POST_BUFFER_SIZE 4096
 
 static void
 send_more_post_data(struct socket *socket)
 {
-	struct connection *conn = socket->conn;
-	struct http_connection_info *http = conn->info;
-	unsigned char buffer[POST_BUFFER_SIZE];
+	struct connection *conn = (struct connection *)socket->conn;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
+	char buffer[POST_BUFFER_SIZE];
 	int got;
 	struct connection_state error;
 
@@ -655,14 +677,14 @@ send_more_post_data(struct socket *socket)
 static void
 http_send_header(struct socket *socket)
 {
-	struct connection *conn = socket->conn;
+	struct connection *conn = (struct connection *)socket->conn;
 	struct http_connection_info *http;
 	int trace = get_opt_bool("protocol.http.trace", NULL);
 	struct string header;
-	unsigned char *post_data = NULL;
+	char *post_data = NULL;
 	struct auth_entry *entry = NULL;
 	struct uri *uri = conn->proxied_uri; /* Set to the real uri */
-	unsigned char *optstr;
+	char *optstr;
 	int use_connect, talking_to_proxy;
 
 	/* Sanity check for a host */
@@ -734,11 +756,11 @@ http_send_header(struct socket *socket)
 
 	/* CONNECT: Proxy-Authorization is intended to be seen by the proxy.  */
 	if (talking_to_proxy) {
-		unsigned char *user = get_opt_str("protocol.http.proxy.user", NULL);
-		unsigned char *passwd = get_opt_str("protocol.http.proxy.passwd", NULL);
+		char *user = get_opt_str("protocol.http.proxy.user", NULL);
+		char *passwd = get_opt_str("protocol.http.proxy.passwd", NULL);
 
 		if (proxy_auth.digest) {
-			unsigned char *response;
+			char *response;
 			int userlen = int_min(strlen(user), AUTH_USER_MAXLEN - 1);
 			int passwordlen = int_min(strlen(passwd), AUTH_PASSWORD_MAXLEN - 1);
 
@@ -762,11 +784,11 @@ http_send_header(struct socket *socket)
 
 		} else {
 			if (user[0]) {
-				unsigned char *proxy_data;
+				char *proxy_data;
 
-				proxy_data = straconcat(user, ":", passwd, (unsigned char *) NULL);
+				proxy_data = straconcat(user, ":", passwd, (char *) NULL);
 				if (proxy_data) {
-					unsigned char *proxy_64 = base64_encode(proxy_data);
+					char *proxy_64 = base64_encode(proxy_data);
 
 					if (proxy_64) {
 						add_to_string(&header, "Proxy-Authorization: Basic ");
@@ -785,7 +807,7 @@ http_send_header(struct socket *socket)
 	 * better error messages.  */
 	optstr = get_opt_str("protocol.http.user_agent", NULL);
 	if (*optstr && strcmp(optstr, " ")) {
-		unsigned char *ustr, ts[64] = "";
+		char *ustr, ts[64] = "";
 		/* TODO: Somehow get the terminal in which the
 		 * document will actually be displayed.  */
 		struct terminal *term = get_default_terminal();
@@ -851,8 +873,13 @@ http_send_header(struct socket *socket)
 	 * sending "Accept: text/css" when it wants an external
 	 * stylesheet, then it should do that only in the inner GET
 	 * and not in the outer CONNECT.  */
-	add_to_string(&header, "Accept: */*");
-	add_crlf_to_string(&header);
+
+	optstr = get_opt_str("protocol.http.accept", NULL);
+	if (optstr[0]) {
+		add_to_string(&header, "Accept: ");
+		add_to_string(&header, optstr);
+		add_crlf_to_string(&header);
+	}
 
 	if (get_opt_bool("protocol.http.compression", NULL))
 		accept_encoding_header(&header);
@@ -875,7 +902,7 @@ http_send_header(struct socket *socket)
 	}
 #ifdef CONFIG_NLS
 	else if (get_opt_bool("protocol.http.accept_ui_language", NULL)) {
-		unsigned char *code = language_to_iso639(current_language);
+		const char *code = language_to_iso639(current_language);
 
 		if (code) {
 			add_to_string(&header, "Accept-Language: ");
@@ -955,7 +982,7 @@ http_send_header(struct socket *socket)
 
 	if (entry) {
 		if (entry->digest) {
-			unsigned char *response;
+			char *response;
 
 			response = get_http_auth_digest_response(entry, uri);
 			if (response) {
@@ -973,13 +1000,13 @@ http_send_header(struct socket *socket)
 			 * and password, separated by a single colon (":")
 			 * character, within a base64 [7] encoded string in the
 			 * credentials. */
-			unsigned char *id;
+			char *id;
 
 			/* Create base64 encoded string. */
 			id = straconcat(entry->user, ":", entry->password,
-					(unsigned char *) NULL);
+					(char *) NULL);
 			if (id) {
-				unsigned char *base64 = base64_encode(id);
+				char *base64 = base64_encode(id);
 
 				mem_free_set(&id, base64);
 			}
@@ -998,7 +1025,7 @@ http_send_header(struct socket *socket)
 		/* We search for first '\n' in uri->post to get content type
 		 * as set by get_form_uri(). This '\n' is dropped if any
 		 * and replaced by correct '\r\n' termination here. */
-		unsigned char *postend = strchr((const char *)uri->post, '\n');
+		char *postend = strchr(uri->post, '\n');
 		struct connection_state error;
 
 		if (postend) {
@@ -1057,8 +1084,8 @@ http_send_header(struct socket *socket)
 #undef POST_BUFFER_SIZE
 
 
-static unsigned char *
-decompress_data(struct connection *conn, unsigned char *data, int len,
+static char *
+decompress_data(struct connection *conn, char *data, int len,
 		int *new_len)
 {
 	*new_len = 0; /* new_len must be zero if we would ever return NULL */
@@ -1109,7 +1136,7 @@ read_more_http_data(struct connection *conn, struct read_buffer *rb,
 static void
 read_http_data_done(struct connection *conn)
 {
-	struct http_connection_info *http = conn->info;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
 
 	/* There's no content but an error so just print
 	 * that instead of nothing. */
@@ -1137,7 +1164,7 @@ read_http_data_done(struct connection *conn)
 static int
 read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 {
-	struct http_connection_info *http = conn->info;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
 	int total_data_len = 0;
 
 	while (1) {
@@ -1169,7 +1196,7 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 			int l = is_line_in_buffer(rb);
 
 			if (l) {
-				unsigned char *de;
+				char *de;
 				int n = 0;
 
 				if (l != -1) {
@@ -1207,7 +1234,7 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 				if (add_fragment(conn->cached, conn->from, rb->data, len) == 1)
 					conn->tries = 0;
 			} else {
-				unsigned char *data = decompress_data(conn, rb->data, len, &data_len);
+				char *data = decompress_data(conn, rb->data, len, &data_len);
 
 				if (add_fragment(conn->cached, conn->from, data, data_len) == 1)
 					conn->tries = 0;
@@ -1258,7 +1285,7 @@ read_chunked_http_data(struct connection *conn, struct read_buffer *rb)
 static int
 read_normal_http_data(struct connection *conn, struct read_buffer *rb)
 {
-	struct http_connection_info *http = conn->info;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
 	int data_len;
 	int len = rb->length;
 
@@ -1275,7 +1302,7 @@ read_normal_http_data(struct connection *conn, struct read_buffer *rb)
 		if (add_fragment(conn->cached, conn->from, rb->data, data_len) == 1)
 			conn->tries = 0;
 	} else {
-		unsigned char *data;
+		char *data;
 finish:
 		data = decompress_data(conn, rb->data, len, &data_len);
 
@@ -1307,8 +1334,8 @@ finish:
 static void
 read_http_data(struct socket *socket, struct read_buffer *rb)
 {
-	struct connection *conn = socket->conn;
-	struct http_connection_info *http = conn->info;
+	struct connection *conn = (struct connection *)socket->conn;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
 	int ret;
 
 	if (socket->state == SOCKET_CLOSED) {
@@ -1385,15 +1412,15 @@ get_header(struct read_buffer *rb)
 /* returns 1 if we need retry the connection (for negotiate-auth only) */
 static int
 check_http_authentication(struct connection *conn, struct uri *uri,
-		unsigned char *header, unsigned char *header_field)
+		char *header, const char *header_field)
 {
-	unsigned char *str, *d;
+	char *str, *d;
 	int ret = 0;
 
 	d = parse_header(header, header_field, &str);
 	while (d) {
 		if (!c_strncasecmp(d, "Basic", 5)) {
-			unsigned char *realm = get_header_param(d, "realm");
+			char *realm = get_header_param(d, "realm");
 
 			if (realm) {
 				add_auth_entry(uri, realm, NULL, NULL, 0);
@@ -1402,9 +1429,9 @@ check_http_authentication(struct connection *conn, struct uri *uri,
 				break;
 			}
 		} else if (!c_strncasecmp(d, "Digest", 6)) {
-			unsigned char *realm = get_header_param(d, "realm");
-			unsigned char *nonce = get_header_param(d, "nonce");
-			unsigned char *opaque = get_header_param(d, "opaque");
+			char *realm = get_header_param(d, "realm");
+			char *nonce = get_header_param(d, "nonce");
+			char *opaque = get_header_param(d, "opaque");
 
 			add_auth_entry(uri, realm, nonce, opaque, 1);
 
@@ -1438,13 +1465,13 @@ check_http_authentication(struct connection *conn, struct uri *uri,
 void
 http_got_header(struct socket *socket, struct read_buffer *rb)
 {
-	struct connection *conn = socket->conn;
-	struct http_connection_info *http = conn->info;
-	unsigned char *head;
+	struct connection *conn = (struct connection *)socket->conn;
+	struct http_connection_info *http = (struct http_connection_info *)conn->info;
+	char *head;
 #ifdef CONFIG_COOKIES
-	unsigned char *cookie, *ch;
+	char *cookie, *ch;
 #endif
-	unsigned char *d;
+	char *d;
 	struct uri *uri = conn->proxied_uri; /* Set to the real uri */
 	struct http_version version = { 0, 9 };
 	struct connection_state state = (!is_in_state(conn->state, S_PROC)
@@ -1602,7 +1629,7 @@ again:
 		}
 
 		if ((d = parse_header(cached->head, "Pragma", NULL))) {
-			if (strstr((const char *)d, "no-cache")) {
+			if (strstr(d, "no-cache")) {
 				cached->cache_mode = CACHE_MODE_NEVER;
 				cached->expire = 0;
 			}
@@ -1611,12 +1638,12 @@ again:
 
 		if (cached->cache_mode != CACHE_MODE_NEVER
 		    && (d = parse_header(cached->head, "Cache-Control", NULL))) {
-			if (strstr((const char *)d, "no-cache") || strstr((const char *)d, "must-revalidate")) {
+			if (strstr(d, "no-cache") || strstr(d, "must-revalidate")) {
 				cached->cache_mode = CACHE_MODE_NEVER;
 				cached->expire = 0;
 
 			} else  {
-				unsigned char *pos = strstr((const char *)d, "max-age=");
+				char *pos = strstr(d, "max-age=");
 
 				assert(cached->cache_mode != CACHE_MODE_NEVER);
 
@@ -1672,13 +1699,13 @@ again:
 
 	}
 	if (h == 407) {
-		unsigned char *str;
+		char *str;
 		int restart = 0;
 
 		d = parse_header(conn->cached->head, "Proxy-Authenticate", &str);
 		while (d) {
 			if (!c_strncasecmp(d, "Basic", 5)) {
-				unsigned char *realm = get_header_param(d, "realm");
+				char *realm = get_header_param(d, "realm");
 
 				if (realm) {
 					mem_free_set(&proxy_auth.realm, realm);
@@ -1688,10 +1715,10 @@ again:
 				}
 
 			} else if (!c_strncasecmp(d, "Digest", 6)) {
-				unsigned char *realm = get_header_param(d, "realm");
-				unsigned char *nonce = get_header_param(d, "nonce");
-				unsigned char *opaque = get_header_param(d, "opaque");
-				unsigned char *stale = get_header_param(d, "stale");
+				char *realm = get_header_param(d, "realm");
+				char *nonce = get_header_param(d, "nonce");
+				char *opaque = get_header_param(d, "opaque");
+				char *stale = get_header_param(d, "stale");
 
 				if (stale) {
 					if (strcasecmp(stale, "true")) restart = 1;
@@ -1761,7 +1788,7 @@ again:
 		foreach (s, conn->downloads) {
 			fprintf(stderr, "conn %p status %p pri %d st %d er %d :: ce %s",
 				conn, s, s->pri, s->state, s->prev_error,
-				s->cached ? s->cached->url : (unsigned char *) "N-U-L-L");
+				s->cached ? s->cached->url : (char *) "N-U-L-L");
 		}
 	}
 #endif
@@ -1774,7 +1801,7 @@ again:
 
 	d = parse_header(conn->cached->head, "Content-Length", NULL);
 	if (d) {
-		unsigned char *ep;
+		char *ep;
 		long long l;
 
 		errno = 0;
@@ -1834,8 +1861,8 @@ again:
 	d = parse_header(conn->cached->head, "ETag", NULL);
 	if (d) {
 		if (conn->cached->etag) {
-			unsigned char *old_tag = conn->cached->etag;
-			unsigned char *new_tag = d;
+			char *old_tag = conn->cached->etag;
+			char *new_tag = d;
 
 			/* http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.19 */
 
@@ -1864,9 +1891,9 @@ again:
 
 	d = parse_header(conn->cached->head, "Content-Encoding", NULL);
 	if (d) {
-#if defined(CONFIG_GZIP) || defined(CONFIG_BZIP2) || defined(CONFIG_LZMA) || defined(CONFIG_BROTLI)
-		unsigned char *extension = get_extension_from_uri(uri);
-		enum stream_encoding file_encoding;
+#if defined(CONFIG_GZIP) || defined(CONFIG_BZIP2) || defined(CONFIG_LZMA) || defined(CONFIG_BROTLI) || defined(CONFIG_ZSTD)
+		char *extension = get_extension_from_uri(uri);
+		stream_encoding_T file_encoding;
 
 		file_encoding = extension ? guess_encoding(extension) : ENCODING_NONE;
 		mem_free_if(extension);
@@ -1878,8 +1905,12 @@ again:
 		if (file_encoding != ENCODING_GZIP
 		    && (!c_strcasecmp(d, "gzip") || !c_strcasecmp(d, "x-gzip")))
 		    	conn->content_encoding = ENCODING_GZIP;
-		if (!c_strcasecmp(d, "deflate") || !c_strcasecmp(d, "x-deflate"))
-			conn->content_encoding = ENCODING_DEFLATE;
+#endif
+
+#ifdef CONFIG_ZSTD
+		if (file_encoding != ENCODING_ZSTD
+		    && (!c_strcasecmp(d, "zstd")))
+			conn->content_encoding = ENCODING_ZSTD;
 #endif
 
 #ifdef CONFIG_BROTLI
