@@ -3,9 +3,13 @@
 
 #include "intl/charsets.h" /* unicode_val_T */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct color_pair;
 struct dialog_data;
-struct box;
+struct el_box;
 struct terminal;
 
 /** How many bytes we need for the colors of one character cell.  */
@@ -24,6 +28,7 @@ struct terminal;
  *
  * XXX: The bold mask is used as part of the color encoding. */
 enum screen_char_attr {
+	SCREEN_ATTR_NONE = 0,
 	SCREEN_ATTR_UNSEARCHABLE = 0x01,
 	SCREEN_ATTR_NODE_NUMBER = 0x02,
 	SCREEN_ATTR_BOLD	= 0x08,
@@ -33,6 +38,8 @@ enum screen_char_attr {
 	SCREEN_ATTR_FRAME	= 0x80,
 };
 
+typedef unsigned short screen_char_attr_T;
+
 /** One position in the terminal screen's image. */
 struct screen_char {
 	/** Contains either character value or frame data.
@@ -219,6 +226,8 @@ enum border_char {
 	BORDER_DSCROSS	  = BD_MIXED(VERTICAL_DOUBLE_AND_HORIZONTAL_SINGLE),
 };
 
+typedef int border_char_T;
+
 /* 0 -> 1 <- 2 v 3 ^ */
 enum border_cross_direction {
 	BORDER_X_RIGHT = 0,
@@ -243,7 +252,7 @@ void draw_char_data(struct terminal *term, int x, int y, unsigned char data);
 
 /** Sets the data to @a border and of a screen position. */
 void draw_border_char(struct terminal *term, int x, int y,
-		      enum border_char border, struct color_pair *color);
+		      border_char_T border, struct color_pair *color);
 
 /** Sets the cross position of two borders. */
 void draw_border_cross(struct terminal *, int x, int y,
@@ -252,11 +261,11 @@ void draw_border_cross(struct terminal *, int x, int y,
 /** Draws a char. */
 #ifdef CONFIG_UTF8
 void draw_char(struct terminal *term, int x, int y,
-	       unicode_val_T data, enum screen_char_attr attr,
+	       unicode_val_T data, int attr,
 	       struct color_pair *color);
 #else
 void draw_char(struct terminal *term, int x, int y,
-	       unsigned char data, enum screen_char_attr attr,
+	       unsigned char data, int attr,
 	       struct color_pair *color);
 #endif /* CONFIG_UTF8 */
 
@@ -264,34 +273,40 @@ void draw_space(struct terminal *term, int x, int y,
 		struct screen_char *color);
 
 /** Draws area defined by @a box using the same colors and attributes. */
-void draw_box(struct terminal *term, struct box *box,
-	      unsigned char data, enum screen_char_attr attr,
+#ifdef CONFIG_UTF8
+void draw_box(struct terminal *term, struct el_box *box,
+	      unicode_val_T data, int attr,
 	      struct color_pair *color);
+#else
+void draw_box(struct terminal *term, struct el_box *box,
+	      unsigned char data, int attr,
+	      struct color_pair *color);
+#endif
 
 /** Draws a shadow of @a width and @a height with color @a color
  * around @a box. */
-void draw_shadow(struct terminal *term, struct box *box,
+void draw_shadow(struct terminal *term, struct el_box *box,
 		 struct color_pair *color, int width, int height);
 
 /** Draw borders. */
-void draw_border(struct terminal *term, struct box *box,
+void draw_border(struct terminal *term, struct el_box *box,
 		 struct color_pair *color, int width);
 
 #ifdef CONFIG_UTF8
-void fix_dwchar_around_box(struct terminal *term, struct box *box, int border,
+void fix_dwchar_around_box(struct terminal *term, struct el_box *box, int border,
 			   int shadow_width, int shadow_height);
 #endif /* CONFIG_UTF8 */
 
 /** Draws @a length chars from @a text. */
 void draw_text(struct terminal *term, int x, int y,
-	       unsigned char *text, int length,
-	       enum screen_char_attr attr,
+	       const char *text, int length,
+	       int attr,
 	       struct color_pair *color);
 
 /** Draws text for dialogs. */
 void draw_dlg_text(struct dialog_data *dlg_data, int x, int y,
-	  unsigned char *text, int length,
-	  enum screen_char_attr attr, struct color_pair *color);
+	  const char *text, int length,
+	  int attr, struct color_pair *color);
 
 
 /** Draws @a length chars from @a line on the screen.  */
@@ -309,4 +324,8 @@ void set_dlg_cursor(struct terminal *term, struct dialog_data *dlg_data, int x,
 /** Blanks the screen. */
 void clear_terminal(struct terminal *);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
