@@ -21,6 +21,9 @@
 #include "terminal/kbd.h"
 #include "terminal/screen.h"
 #include "terminal/terminal.h"
+#ifdef CONFIG_TERMINFO
+#include "terminal/terminfo.h"
+#endif
 #include "util/conv.h"
 #include "util/error.h"
 #include "util/memory.h"
@@ -31,11 +34,25 @@
  * Please mention ELinks bug 96 in commit logs.  --KON */
 
 /** Mapping from (enum ::border_char - 0xB0) to ASCII characters.  */
-const unsigned char frame_dumb[48] =	"   ||||++||++++++--|-+||++--|-+----++++++++     ";
+const unsigned char frame_dumb[48] = {
+	' ',' ',' ','|','|','|','|','+',
+	'+','|','|','+','+','+','+','+',
+	'+','-','-','|','-','+','|','|',
+	'+','+','-','-','|','-','+','-',
+	'-','-','-','+','+','+','+','+',
+	'+','+','+',' ',' ',' ',' ',' '
+};
 
 /** Mapping from (enum ::border_char - 0xB0) to VT100 line-drawing
  * characters.  */
-static const unsigned char frame_vt100[48] =	"aaaxuuukkuxkjjjkmvwtqnttmlvwtqnvvwwmmllnnjla    ";
+static const unsigned char frame_vt100[48] = {
+	'a','a','a','x','u','u','u','k',
+	'k','u','x','k','j','j','j','k',
+	'm','v','w','t','q','n','t','t',
+	'm','l','v','w','t','q','n','v',
+	'v','w','w','m','m','l','l','n',
+	'n','j','l','a',' ',' ',' ',' '
+};
 
 /** Mapping from (enum ::border_char - 0xB0) to VT100 line-drawing
  * characters encoded in CP437.
@@ -115,7 +132,7 @@ static const unsigned char frame_freebsd[48] = {
 /** Mapping from (enum ::border_char - 0xB0) to obsolete FreeBSD ACS
  * graphics encoded in CP437.
  * When UTF-8 I/O is enabled, ELinks uses this array instead of
- * ::frame_freebsd[], and converts the characters from CP437 to UTF-8.
+  * ::frame_freebsd[], and converts the characters from CP437 to UTF-8.
  *
  * Derived from ::frame_freebsd[] by converting the characters to
  * Unicode and back to CP437.  frame_freebsd[1] = 138 = 0x8a = U+240B
@@ -227,7 +244,7 @@ struct screen_driver_opt {
 	const struct string *underline;
 
 	/** The color mode */
-	enum color_mode color_mode;
+	color_mode_T color_mode;
 
 #if defined(CONFIG_88_COLORS) || defined(CONFIG_256_COLORS)
 	const struct string *color256_seqs;
@@ -246,6 +263,11 @@ struct screen_driver_opt {
 	/* Whether the terminal supports combining characters. */
 	unsigned int combine:1;
 #endif /* CONFIG_COMBINE */
+
+#ifdef CONFIG_TERMINFO
+	/* Whether use terminfo. */
+	unsigned int terminfo:1;
+#endif
 };
 
 /** Used in @c add_char*() and @c redraw_screen() to reduce the logic.
@@ -258,13 +280,13 @@ struct screen_driver {
 
 	/** The terminal._template_.type. Together with the #name member they
 	 * uniquely identify the screen_driver. */
-	enum term_mode_type type;
+	term_mode_type_T type;
 
 	/** set_screen_driver_opt() sets these.  */
 	struct screen_driver_opt opt;
 
 	/* The terminal._template_ name. */
-	unsigned char name[1]; /* XXX: Keep last! */
+	char name[1]; /* XXX: Keep last! */
 };
 
 /** Default options for ::TERM_DUMB.  */
@@ -285,6 +307,9 @@ static const struct screen_driver_opt dumb_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for ::TERM_VT100.  */
@@ -305,6 +330,9 @@ static const struct screen_driver_opt vt100_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for ::TERM_LINUX.  */
@@ -325,6 +353,9 @@ static const struct screen_driver_opt linux_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for ::TERM_KOI8.  */
@@ -345,6 +376,9 @@ static const struct screen_driver_opt koi8_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for ::TERM_FREEBSD.  */
@@ -365,6 +399,9 @@ static const struct screen_driver_opt freebsd_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for ::TERM_FBTERM.  */
@@ -385,10 +422,13 @@ static const struct screen_driver_opt fbterm_screen_driver_opt = {
 #ifdef CONFIG_COMBINE
 	/* combine */		0,
 #endif /* CONFIG_COMBINE */
+#ifdef CONFIG_TERMINFO
+	/* terminfo */		0,
+#endif
 };
 
 /** Default options for all the different types of terminals.
- * XXX: Keep in sync with enum term_mode_type. */
+ * XXX: Keep in sync with term_mode_type_T. */
 static const struct screen_driver_opt *const screen_driver_opts[] = {
 	/* TERM_DUMB: */	&dumb_screen_driver_opt,
 	/* TERM_VT100: */	&vt100_screen_driver_opt,
@@ -501,16 +541,58 @@ set_screen_driver_opt(struct screen_driver *driver, struct option *term_spec)
 		} else if (driver->type == TERM_VT100) {
 			driver->opt.frame = frame_vt100;
 		}
- 	} /* !utf8_io */
+	} /* !utf8_io */
+#ifdef CONFIG_TERMINFO
+	driver->opt.terminfo = get_cmd_opt_bool("terminfo");
+	if (!driver->opt.terminfo) {
+		return;
+	} 
+#ifdef CONFIG_TRUE_COLOR
+	if (driver->opt.color_mode == COLOR_MODE_TRUE_COLOR) {
+		driver->opt.terminfo = 0;
+		return;
+	}
+#endif
+	if (driver->opt.color_mode == COLOR_MODE_MONO) {
+		driver->opt.terminfo = 0;
+		return;
+	}
+
+	switch (terminfo_max_colors()) {
+	case 88:
+#ifdef CONFIG_88_COLORS
+		driver->opt.color_mode = COLOR_MODE_88;
+#else
+		driver->opt.color_mode = COLOR_MODE_16;
+#endif
+		break;
+
+	case 256:
+#ifdef CONFIG_256_COLORS
+		driver->opt.color_mode = COLOR_MODE_256;
+#else
+		driver->opt.color_mode = COLOR_MODE_16;
+#endif
+		break;
+
+	case 16:
+	case 8:
+		driver->opt.color_mode = COLOR_MODE_16;
+		break;
+	default:
+		driver->opt.color_mode = COLOR_MODE_MONO;
+		break;
+	}
+#endif
 }
 
 static int
 screen_driver_change_hook(struct session *ses, struct option *term_spec,
 			  struct option *changed)
 {
-	enum term_mode_type type = get_opt_int_tree(term_spec, "type", NULL);
+	term_mode_type_T type = get_opt_int_tree(term_spec, "type", NULL);
 	struct screen_driver *driver;
-	unsigned char *name = term_spec->name;
+	const char *name = term_spec->name;
 
 	foreach (driver, active_screen_drivers)
 		if (driver->type == type && !strcmp(driver->name, name)) {
@@ -522,12 +604,12 @@ screen_driver_change_hook(struct session *ses, struct option *term_spec,
 }
 
 static inline struct screen_driver *
-add_screen_driver(enum term_mode_type type, struct terminal *term, int env_len)
+add_screen_driver(term_mode_type_T type, struct terminal *term, int env_len)
 {
 	struct screen_driver *driver;
 
 	/* One byte is reserved for name in struct screen_driver. */
-	driver = mem_alloc(sizeof(*driver) + env_len);
+	driver = (struct screen_driver *)mem_alloc(sizeof(*driver) + env_len);
 	if (!driver) return NULL;
 
 	/* These four operations fully initialize *driver.  */
@@ -549,8 +631,8 @@ add_screen_driver(enum term_mode_type type, struct terminal *term, int env_len)
 static inline struct screen_driver *
 get_screen_driver(struct terminal *term)
 {
-	enum term_mode_type type = get_opt_int_tree(term->spec, "type", NULL);
-	unsigned char *name = term->spec->name;
+	term_mode_type_T type = get_opt_int_tree(term->spec, "type", NULL);
+	const char *name = term->spec->name;
 	int len = strlen(name);
 	struct screen_driver *driver;
 
@@ -584,8 +666,14 @@ done_screen_drivers(struct module *xxx)
 static inline struct string *
 add_cursor_move_to_string(struct string *screen, int y, int x)
 {
+#ifdef CONFIG_TERMINFO
+	if (get_cmd_opt_bool("terminfo")) {
+		const char *aa = terminfo_cursor_address(y-1, x-1);
+		return add_to_string(screen, aa);
+	} else
+#endif
 #define CURSOR_NUM_LEN 10 /* 10 chars for @y and @x numbers should be more than enough. */
-	unsigned char code[4 + 2 * CURSOR_NUM_LEN + 1];
+{	char code[4 + 2 * CURSOR_NUM_LEN + 1];
 	unsigned int length = 2;
 
 	code[0] = '\033';
@@ -602,6 +690,7 @@ add_cursor_move_to_string(struct string *screen, int y, int x)
 	code[length++] = 'H';
 
 	return add_bytes_to_string(screen, code, length);
+}
 #undef CURSOR_NUM_LEN
 }
 
@@ -806,7 +895,14 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	    italic != state->italic && driver->opt.italic
 	   ) {
 		state->italic = italic;
-		add_term_string(screen, driver->opt.italic[!!italic]);
+#ifdef CONFIG_TERMINFO
+		if (driver->opt.terminfo) {
+			add_to_string(screen, terminfo_set_italics(italic));
+		} else
+#endif
+		{
+			add_term_string(screen, driver->opt.italic[!!italic]);
+		}
 	}
 
 	if (
@@ -816,7 +912,14 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	    underline != state->underline && driver->opt.underline
 	   ) {
 		state->underline = underline;
-		add_term_string(screen, driver->opt.underline[!!underline]);
+#ifdef CONFIG_TERMINFO
+		if (driver->opt.terminfo) {
+			add_to_string(screen, terminfo_set_underline(underline));
+		} else
+#endif
+		{
+			add_term_string(screen, driver->opt.underline[!!underline]);
+		}
 	}
 
 	if (
@@ -827,7 +930,14 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	   ) {
 		state->bold = bold;
 		if (bold) {
-			add_bytes_to_string(screen, "\033[1m", 4);
+#ifdef CONFIG_TERMINFO
+			if (driver->opt.terminfo) {
+				add_to_string(screen, terminfo_set_bold(bold));
+			} else
+#endif
+			{
+				add_bytes_to_string(screen, "\033[1m", 4);
+			}
 		} else {
 			/* Force repainting of the other attributes. */
 			state->color[0] = ch->c.color[0] + 1;
@@ -842,49 +952,65 @@ add_char16(struct string *screen, struct screen_driver *driver,
 	   ) {
 		copy_color_16(state->color, ch->c.color);
 
-		add_bytes_to_string(screen, "\033[0", 3);
+#ifdef CONFIG_TERMINFO
+		if (driver->opt.terminfo) {
+			add_to_string(screen, terminfo_set_bold(bold));
+			add_to_string(screen, terminfo_set_foreground(TERM_COLOR_FOREGROUND_16(ch->c.color)));
+			add_to_string(screen, terminfo_set_background(TERM_COLOR_BACKGROUND_16(ch->c.color)));
 
-		/* @set_screen_driver_opt has set @driver->opt.color_mode
-		 * according to terminal-type-specific options.
-		 * The caller of @add_char16 has already partially
-		 * checked it, but there are still these possibilities:
-		 * - COLOR_MODE_MONO.  Then don't show colors, but
-		 *   perhaps use the standout attribute.
-		 * - COLOR_MODE_16.  Use 16 colors.
-		 * - An unsupported color mode.  Use 16 colors.  */
-		if (driver->opt.color_mode != COLOR_MODE_MONO) {
-			unsigned char code[6] = ";30;40";
-			unsigned char bgcolor = TERM_COLOR_BACKGROUND_16(ch->c.color);
+			if (italic)
+				add_to_string(screen, terminfo_set_italics(italic));
 
-			code[2] += TERM_COLOR_FOREGROUND_16(ch->c.color);
+			if (underline)
+				add_to_string(screen, terminfo_set_underline(underline));
 
-			if (!driver->opt.transparent || bgcolor != 0) {
-				code[5] += bgcolor;
-				add_bytes_to_string(screen, code, 6);
-			} else {
-				add_bytes_to_string(screen, code, 3);
+		} else
+#endif
+		{
+			add_bytes_to_string(screen, "\033[0", 3);
+
+			/* @set_screen_driver_opt has set @driver->opt.color_mode
+			 * according to terminal-type-specific options.
+			 * The caller of @add_char16 has already partially
+			 * checked it, but there are still these possibilities:
+			 * - COLOR_MODE_MONO.  Then don't show colors, but
+			 *   perhaps use the standout attribute.
+			 * - COLOR_MODE_16.  Use 16 colors.
+			 * - An unsupported color mode.  Use 16 colors.  */
+			if (driver->opt.color_mode != COLOR_MODE_MONO) {
+				char code[] = ";30;40";
+				unsigned char bgcolor = TERM_COLOR_BACKGROUND_16(ch->c.color);
+
+				code[2] += TERM_COLOR_FOREGROUND_16(ch->c.color);
+
+				if (!driver->opt.transparent || bgcolor != 0) {
+					code[5] += bgcolor;
+					add_bytes_to_string(screen, code, 6);
+				} else {
+					add_bytes_to_string(screen, code, 3);
+				}
+
+			} else if (ch->attr & SCREEN_ATTR_STANDOUT) {
+				/* Flip the fore- and background colors for highlighing
+				 * purposes. */
+				add_bytes_to_string(screen, ";7", 2);
 			}
 
-		} else if (ch->attr & SCREEN_ATTR_STANDOUT) {
-			/* Flip the fore- and background colors for highlighing
-			 * purposes. */
-			add_bytes_to_string(screen, ";7", 2);
-		}
+			if (italic && driver->opt.italic) {
+				add_bytes_to_string(screen, ";3", 2);
+			}
 
-		if (italic && driver->opt.italic) {
-			add_bytes_to_string(screen, ";3", 2);
-		}
+			if (underline && driver->opt.underline) {
+				add_bytes_to_string(screen, ";4", 2);
+			}
 
-		if (underline && driver->opt.underline) {
-			add_bytes_to_string(screen, ";4", 2);
-		}
+			/* Check if the char should be rendered bold. */
+			if (bold) {
+				add_bytes_to_string(screen, ";1", 2);
+			}
 
-		/* Check if the char should be rendered bold. */
-		if (bold) {
-			add_bytes_to_string(screen, ";1", 2);
+			add_bytes_to_string(screen, "m", 1);
 		}
-
-		add_bytes_to_string(screen, "m", 1);
 	}
 
 	add_char_data(screen, driver, ch->data, border);
@@ -894,8 +1020,8 @@ add_char16(struct string *screen, struct screen_driver *driver,
 static inline void
 add_char_color(struct string *screen, const struct string *seq, unsigned char color)
 {
-	unsigned char color_buf[3];
-	unsigned char *color_pos = color_buf;
+	char color_buf[3];
+	char *color_pos = color_buf;
 	int seq_pos = 0;
        	int color_len = 1;
 
@@ -961,17 +1087,38 @@ add_char256(struct string *screen, struct screen_driver *driver,
 
 		if ((attr_delta & SCREEN_ATTR_ITALIC) && driver->opt.italic) {
 			state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
-			add_term_string(screen, driver->opt.italic[state->italic]);
+#ifdef CONFIG_TERMINFO
+			if (driver->opt.terminfo) {
+				add_to_string(screen, terminfo_set_italics(state->italic));
+			} else
+#endif
+			{
+				add_term_string(screen, driver->opt.italic[state->italic]);
+			}
 		}
 
 		if ((attr_delta & SCREEN_ATTR_UNDERLINE) && driver->opt.underline) {
 			state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
-			add_term_string(screen, driver->opt.underline[state->underline]);
+#ifdef CONFIG_TERMINFO
+			if (driver->opt.terminfo) {
+				add_to_string(screen, terminfo_set_underline(state->underline));
+			} else
+#endif
+			{
+				add_term_string(screen, driver->opt.underline[state->underline]);
+			}
 		}
 
 		if (attr_delta & SCREEN_ATTR_BOLD) {
 			if (ch->attr & SCREEN_ATTR_BOLD) {
-				add_bytes_to_string(screen, "\033[1m", 4);
+#ifdef CONFIG_TERMINFO
+				if (driver->opt.terminfo) {
+					add_to_string(screen, terminfo_set_bold(1));
+				} else
+#endif
+				{
+					add_bytes_to_string(screen, "\033[1m", 4);
+				}
 			} else {
 				/* Force repainting of the other attributes. */
 				state->color[0] = ch->c.color[0] + 1;
@@ -988,23 +1135,41 @@ add_char256(struct string *screen, struct screen_driver *driver,
 	    !compare_color_256(ch->c.color, state->color)
 	   ) {
 		copy_color_256(state->color, ch->c.color);
+#ifdef CONFIG_TERMINFO
+		if (driver->opt.terminfo) {
+			add_to_string(screen, terminfo_set_bold(ch->attr & SCREEN_ATTR_BOLD));
+			add_to_string(screen, terminfo_set_foreground(ch->c.color[0]));
+			add_to_string(screen, terminfo_set_background(ch->c.color[1]));
 
-		add_foreground_color(screen, driver->opt.color256_seqs, ch);
-		if (!driver->opt.transparent || ch->c.color[1] != 0) {
-			add_background_color(screen, driver->opt.color256_seqs, ch);
-		}
+			if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
+				state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
+				add_to_string(screen, terminfo_set_italics(state->italic));
+			}
 
-		if (ch->attr & SCREEN_ATTR_BOLD)
-			add_bytes_to_string(screen, "\033[1m", 4);
+			if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
+				state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
+				add_to_string(screen, terminfo_set_underline(state->underline));
+			}
+		} else
+#endif
+		{
+			add_foreground_color(screen, driver->opt.color256_seqs, ch);
+			if (!driver->opt.transparent || ch->c.color[1] != 0) {
+				add_background_color(screen, driver->opt.color256_seqs, ch);
+			}
 
-		if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
-			state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
-			add_term_string(screen, driver->opt.italic[state->italic]);
-		}
+			if (ch->attr & SCREEN_ATTR_BOLD)
+				add_bytes_to_string(screen, "\033[1m", 4);
 
-		if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
-			state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
-			add_term_string(screen, driver->opt.underline[state->underline]);
+			if (ch->attr & SCREEN_ATTR_ITALIC && driver->opt.italic) {
+				state->italic = !!(ch->attr & SCREEN_ATTR_ITALIC);
+				add_term_string(screen, driver->opt.italic[state->italic]);
+			}
+
+			if (ch->attr & SCREEN_ATTR_UNDERLINE && driver->opt.underline) {
+				state->underline = !!(ch->attr & SCREEN_ATTR_UNDERLINE);
+				add_term_string(screen, driver->opt.underline[state->underline]);
+			}
 		}
 	}
 
@@ -1022,13 +1187,13 @@ static const struct string color_true_seqs[] = {
 static inline void
 add_char_true_color(struct string *screen, const struct string *seq, unsigned char *colors)
 {
-	unsigned char color_buf[3];
+	char color_buf[3];
 	int i;
 
 	check_string_magic(seq);
 	add_string_to_string(screen, seq);
 	for (i = 0; i < 3; i++) {
-		unsigned char *color_pos = color_buf;
+		char *color_pos = color_buf;
 		int color_len = 1;
 		unsigned char color = colors[i];
 
@@ -1141,19 +1306,19 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 {										\
 	struct terminal_screen *screen = (term_)->screen;			\
 	int y = screen->dirty_from;					\
-	int ypos = y * (term_)->width;						\
-	int prev_y = -1;							\
 	int xmax = (term_)->width - 1;						\
 	int ymax = (term_)->height - 1;						\
-	struct screen_char *current = &screen->last_image[ypos];	\
-	struct screen_char *pos = &screen->image[ypos];		\
-	struct screen_char *prev_pos = NULL; /* Warning prevention. */	\
 										\
 	int_upper_bound(&screen->dirty_to, ymax);				\
 										\
 	for (; y <= screen->dirty_to; y++) {					\
+		int ypos = y * (term_)->width;					\
+		struct screen_char *current = &screen->last_image[ypos];	\
+		struct screen_char *pos = &screen->image[ypos];			\
+		struct screen_char *start_of_line = pos;			\
 		int is_last_line = (y == ymax);					\
 		int x = 0;						\
+		int dirty = 0;						\
 										\
 		for (; x <= xmax; x++, current++, pos++) {			\
 			/*  Workaround for terminals without
@@ -1181,19 +1346,18 @@ add_char_true(struct string *screen, struct screen_driver *driver,
 				    && pos->attr == current->attr)		\
 					continue;				\
 			}							\
-										\
-			/* Move the cursor when @prev_pos is more than 10 chars
-			 * away. */						\
-			if (prev_y != y || prev_pos + 10 <= pos) {		\
-				add_cursor_move_to_string(image_, y + 1, x + 1);\
-				prev_pos = pos;					\
-				prev_y = y;					\
-			}							\
-				 						\
-			for (; prev_pos <= pos ; prev_pos++)			\
-				ADD_CHAR(image_, driver_, prev_pos, state_);	\
+			dirty = 1;						\
+			break;							\
 		}								\
-	}									\
+										\
+		if (!dirty) {							\
+			continue;						\
+		}								\
+		add_cursor_move_to_string(image_, y + 1, 1);			\
+		for (pos = start_of_line, x = 0; x <= xmax; x++, pos++) {	\
+			ADD_CHAR(image_, driver_, pos, state_);	\
+		}							\
+	}								\
 }
 
 /*! Updating of the terminal screen is done by checking what needs to
@@ -1288,6 +1452,12 @@ erase_screen(struct terminal *term)
 		want_draw();
 	}
 
+#ifdef CONFIG_TERMINFO
+	if (get_cmd_opt_bool("terminfo")) {
+		const char *text = terminfo_clear_screen();
+		hard_write(term->fdout, text, strlen(text));
+	} else 
+#endif
 	hard_write(term->fdout, "\033[2J\033[1;1H", 10);
 	if (term->master) done_draw();
 }
@@ -1307,7 +1477,7 @@ init_screen(void)
 {
 	struct terminal_screen *screen;
 
-	screen = mem_calloc(1, sizeof(*screen));
+	screen = (struct terminal_screen *)mem_calloc(1, sizeof(*screen));
 	if (!screen) return NULL;
 
 	screen->lcx = -1;
@@ -1338,7 +1508,7 @@ resize_screen(struct terminal *term, int width, int height)
 
 	bsize = size * sizeof(*image);
 
-	image = mem_realloc(screen->image, bsize * 2);
+	image = (struct screen_char *)mem_realloc(screen->image, bsize * 2);
 	if (!image) return;
 
 	screen->image = image;
