@@ -17,13 +17,14 @@
 #include "cache/cache.h"
 #include "document/document.h"
 #include "document/html/frames.h"
+#include "document/html/iframes.h"
 #include "document/options.h"
 #include "document/refresh.h"
 #include "document/renderer.h"
 #include "document/view.h"
 #include "dialogs/status.h"		/* print_screen_status() */
 #include "intl/charsets.h"
-#include "intl/gettext/libintl.h"
+#include "intl/libintl.h"
 #include "protocol/uri.h"
 #include "session/location.h"
 #include "session/session.h"
@@ -108,7 +109,7 @@ draw_frame_lines(struct terminal *term, struct frameset_desc *frameset_desc,
 			int width = frameset_desc->frame_desc[i].width;
 
 			if (i) {
-				struct box box;
+				struct el_box box;
 
 				set_box(&box, x, y + 1, 1, height);
 				draw_box(term, &box, BORDER_SVLINE, SCREEN_ATTR_FRAME, colors);
@@ -123,7 +124,7 @@ draw_frame_lines(struct terminal *term, struct frameset_desc *frameset_desc,
 			}
 
 			if (j) {
-				struct box box;
+				struct el_box box;
 
 				set_box(&box, x + 1, y, width, 1);
 				draw_box(term, &box, BORDER_SHLINE, SCREEN_ATTR_FRAME, colors);
@@ -165,6 +166,94 @@ draw_frame_lines(struct terminal *term, struct frameset_desc *frameset_desc,
 	}
 }
 
+static void
+draw_iframe_lines(struct terminal *term, struct iframeset_desc *iframe_desc,
+		 int xp, int yp, struct color_pair *colors)
+{
+	int j;
+
+	assert(term && iframe_desc && iframe_desc->iframe_desc);
+	if_assert_failed return;
+
+	for (j = 0; j < iframe_desc->n; j++) {
+		struct el_box box;
+
+		int y = iframe_desc->iframe_desc[j].y - 1;
+		int x = iframe_desc->iframe_desc[j].x - 1;
+
+		int height = iframe_desc->iframe_desc[j].height + 1;
+		int width = iframe_desc->iframe_desc[j].width + 1;
+
+		set_box(&box, x, y + 1, 1, height - 1);
+		draw_box(term, &box, BORDER_SVLINE, SCREEN_ATTR_FRAME, colors);
+
+		set_box(&box, x + width, y + 1, 1, height - 1);
+		draw_box(term, &box, BORDER_SVLINE, SCREEN_ATTR_FRAME, colors);
+
+		set_box(&box, x + 1, y, width -1 , 1);
+		draw_box(term, &box, BORDER_SHLINE, SCREEN_ATTR_FRAME, colors);
+
+		set_box(&box, x + 1, y + height, width - 1, 1);
+		draw_box(term, &box, BORDER_SHLINE, SCREEN_ATTR_FRAME, colors);
+
+		draw_border_char(term, x, y, BORDER_SULCORNER, colors);
+		draw_border_char(term, x, y + height, BORDER_SDLCORNER, colors);
+
+		draw_border_char(term, x + width, y, BORDER_SURCORNER, colors);
+		draw_border_char(term, x + width, y + height, BORDER_SDRCORNER, colors);
+		draw_border_cross(term, x, y, BORDER_X_DOWN, colors);
+	}
+}
+
+
+static void
+draw_clipboard(struct terminal *term, struct document_view *doc_view)
+{
+	struct document *document = doc_view->document;
+	struct color_pair *color;
+	int starty, startx, endy, endx, x, y, xoffset, yoffset;
+
+	assert(term && doc_view);
+	if_assert_failed return;
+
+	if (document->clipboard_status == CLIPBOARD_NONE) {
+		return;
+	}
+
+	color = get_bfu_color(term, "clipboard");
+	xoffset = doc_view->box.x - doc_view->vs->x;
+	yoffset = doc_view->box.y - doc_view->vs->y;
+
+
+	if (document->clipboard_box.height >= 0) {
+		starty = int_max(doc_view->box.y, document->clipboard_box.y + yoffset);
+		endy = int_min(doc_view->box.y + doc_view->box.height,
+			document->clipboard_box.y + document->clipboard_box.height + yoffset);
+	} else {
+		endy = int_max(doc_view->box.y, document->clipboard_box.y + yoffset);
+		starty = int_min(doc_view->box.y + doc_view->box.height,
+			document->clipboard_box.y + document->clipboard_box.height + yoffset);
+	}
+
+	if (document->clipboard_box.width >= 0) {
+		startx =  int_max(doc_view->box.x, document->clipboard_box.x + xoffset);
+		endx = int_min(doc_view->box.x + doc_view->box.width,
+			document->clipboard_box.x + document->clipboard_box.width + xoffset);
+	} else {
+		endx =  int_max(doc_view->box.x, document->clipboard_box.x + xoffset);
+		startx = int_min(doc_view->box.x + doc_view->box.width,
+			document->clipboard_box.x + document->clipboard_box.width + xoffset);
+	}
+
+	for (y = starty; y <= endy; ++y) {
+		for (x = startx; x <= endx; ++x) {
+			draw_char_color(term, x, y, color);
+		}
+	}
+	doc_view->last_x = doc_view->last_y = -1;
+
+}
+
 static void
 draw_view_status(struct session *ses, struct document_view *doc_view, int active)
 {
@@ -173,6 +262,7 @@ draw_view_status(struct session *ses, struct document_view *doc_view, int active
 	draw_forms(term, doc_view);
 	if (active) {
 		draw_searched(term, doc_view);
+		draw_clipboard(term, doc_view);
 		draw_current_link(ses, doc_view);
 	}
 }
@@ -184,7 +274,7 @@ check_link_under_cursor(struct session *ses, struct document_view *doc_view)
 {
 	int x = ses->tab->x;
 	int y = ses->tab->y;
-	struct box *box = &doc_view->box;
+	struct el_box *box = &doc_view->box;
 	struct link *link;
 
 	link = get_link_at_coordinates(doc_view, x - box->x, y - box->y);
@@ -202,7 +292,7 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	struct color_pair color;
 	struct view_state *vs;
 	struct terminal *term;
-	struct box *box;
+	struct el_box *box;
 	struct screen_char *last = NULL;
 
 	int vx, vy;
@@ -236,21 +326,31 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 
 	vs = doc_view->vs;
 	if (!vs) {
-		draw_box(term, box, ' ', 0, &color);
+		int bgchar = get_opt_int("ui.background_char", ses);
+#ifdef CONFIG_UTF8
+		draw_box(term, box, bgchar, 0, get_bfu_color(term, "desktop"));
+#else
+		draw_box(term, box, (unsigned char)bgchar, 0, get_bfu_color(term, "desktop"));
+#endif
 		return;
 	}
 
 	if (document_has_frames(doc_view->document)) {
-	 	draw_box(term, box, ' ', 0, &color);
+		int bgchar = get_opt_int("ui.background_char", ses);
+#ifdef CONFIG_UTF8
+		draw_box(term, box, bgchar, 0, get_bfu_color(term, "desktop"));
+#else
+		draw_box(term, box, (unsigned char)bgchar, 0, get_bfu_color(term, "desktop"));
+#endif
 		draw_frame_lines(term, doc_view->document->frame_desc, box->x, box->y, &color);
 		if (vs->current_link == -1)
 			vs->current_link = 0;
 		return;
 	}
 
+
 	if (ses->navigate_mode == NAVIGATE_LINKWISE) {
 		check_vs(doc_view);
-
 	} else {
 		check_link_under_cursor(ses, doc_view);
 	}
@@ -263,6 +363,22 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 			doc_view->vs->y = vy;
 			set_link(doc_view);
 		}
+		if (vy == -1) {
+			struct location *loc = cur_loc(ses);
+
+			if (loc) {
+				struct uri *cur_uri = loc->vs.uri;
+
+				if (list_has_prev(ses->history.history, loc)) {
+					struct uri *prev_uri = loc->prev->vs.uri;
+
+					if (compare_uri(cur_uri, prev_uri, URI_BASE)) {
+						doc_view->vs->y = doc_view->prev_y;
+						set_link(doc_view);
+					}
+				}
+			}
+		}
 	}
 	vx = vs->x;
 	vy = vs->y;
@@ -276,7 +392,13 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	}
 	doc_view->last_x = vx;
 	doc_view->last_y = vy;
-	draw_box(term, box, ' ', 0, &color);
+
+	int bgchar = get_opt_int("ui.background_char", ses);
+#ifdef CONFIG_UTF8
+	draw_box(term, box, bgchar, 0, get_bfu_color(term, "desktop"));
+#else
+	draw_box(term, box, (unsigned char)bgchar, 0, get_bfu_color(term, "desktop"));
+#endif
 	if (!doc_view->document->height) return;
 
 	while (vs->y >= doc_view->document->height) vs->y -= box->height;
@@ -291,16 +413,24 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	     y++) {
 		struct screen_char *first = NULL;
 		int i, j;
+		int last_index = 0;
 		int st = int_max(vx, 0);
 		int en = int_min(doc_view->document->data[y].length,
 				 box->width + vx);
-		int max = int_min(en, st + 30);
+		int max = int_min(en, st + 200);
 
 		if (en - st > 0) {
 			draw_line(term, box->x + st - vx, box->y + y - vy,
 				  en - st,
 				  &doc_view->document->data[y].chars[st]);
-			last = &doc_view->document->data[y].chars[en - 1];
+
+			for (i = en - 1; i >= 0; --i) {
+				if (doc_view->document->data[y].chars[i].data != ' ') {
+					last = &doc_view->document->data[y].chars[i];
+					last_index = i + 1;
+					break;
+				}
+			}
 		}
 		for (i = st; i < max; i++) {
 			if (doc_view->document->data[y].chars[i].data != ' ') {
@@ -308,11 +438,13 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 				break;
 			}
 		}
-		for (j = st + 1; j < i; j++) {
+
+		for (j = st; j < i; j++) {
 			draw_space(term, box->x + j - vx, box->y + y - vy,
 				   first);
 		}
-		for (i = en ? en : 1; i < box->width + vx - 1; i++) {
+
+		for (i = last_index; i < box->width + vx; i++) {
 			draw_space(term, box->x + i - vx, box->y + y - vy,
 				   last);
 		}
@@ -320,6 +452,12 @@ draw_doc(struct session *ses, struct document_view *doc_view, int active)
 	draw_view_status(ses, doc_view, active);
 	if (has_search_word(doc_view))
 		doc_view->last_x = doc_view->last_y = -1;
+
+	if (document_has_iframes(doc_view->document)) {
+		draw_iframe_lines(term, doc_view->document->iframe_desc, box->x, box->y, &color);
+		if (vs->current_link == -1)
+			vs->current_link = 0;
+	}
 }
 
 static void
@@ -332,13 +470,19 @@ draw_frames(struct session *ses)
 	assert(ses && ses->doc_view && ses->doc_view->document);
 	if_assert_failed return;
 
-	if (!document_has_frames(ses->doc_view->document)) return;
+	if (!document_has_frames(ses->doc_view->document)
+	&& !document_has_iframes(ses->doc_view->document)) return;
 
 	n = 0;
 	foreach (doc_view, ses->scrn_frames) {
 	       doc_view->last_x = doc_view->last_y = -1;
 	       n++;
 	}
+	foreach (doc_view, ses->scrn_iframes) {
+	       doc_view->last_x = doc_view->last_y = -1;
+	       n++;
+	}
+
 	l = &cur_loc(ses)->vs.current_link;
 	*l = int_max(*l, 0) % int_max(n, 1);
 
@@ -353,6 +497,9 @@ draw_frames(struct session *ses)
 			else if (doc_view->depth > d)
 				more = 1;
 		}
+		if (d == 0) foreach (doc_view, ses->scrn_iframes) {
+			draw_doc(ses, doc_view, doc_view == current_doc_view);
+		}
 
 		if (!more) break;
 		d++;
@@ -381,12 +528,17 @@ draw_formatted(struct session *ses, int rerender)
 
 	if (!ses->doc_view || !ses->doc_view->document) {
 		/*INTERNAL("document not formatted");*/
-		struct box box;
+		struct el_box box;
+		int bgchar = get_opt_int("ui.background_char", ses);
 
 		set_box(&box, 0, 1,
 			ses->tab->term->width,
 			ses->tab->term->height - 2);
-		draw_box(ses->tab->term, &box, ' ', 0, NULL);
+#ifdef CONFIG_UTF8
+		draw_box(ses->tab->term, &box, bgchar, 0, get_bfu_color(ses->tab->term, "desktop"));
+#else
+		draw_box(ses->tab->term, &box, (unsigned char)bgchar, 0, get_bfu_color(ses->tab->term, "desktop"));
+#endif
 		return;
 	}
 
