@@ -6,6 +6,10 @@
 #include "document/view.h"
 #include "viewer/action.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct module;
 struct session;
 struct terminal;
@@ -18,7 +22,7 @@ enum frame_event_status find_next(struct session *ses, struct document_view *doc
 enum frame_event_status move_search_next(struct session *ses, struct document_view *doc_view);
 enum frame_event_status move_search_prev(struct session *ses, struct document_view *doc_view);
 
-
+void search_for(struct session *ses, const char *str);
 enum frame_event_status search_dlg(struct session *ses, struct document_view *doc_view, int direction);
 enum frame_event_status search_typeahead(struct session *ses, struct document_view *doc_view, action_id_T action_id);
 
@@ -29,4 +33,8 @@ static inline int has_search_word(struct document_view *doc_view)
 		&& (*doc_view->search_word)[0]);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
